@@ -3,6 +3,7 @@ use std::path::Path;
 
 use serde::ser;
 
+use crate::util::ProcessBuilder;
 use crate::util::{CargoResult, CargoResultExt, Config, RustfixDiagnosticServer};
 
 /// Configuration information for a rustc build.
@@ -23,12 +24,8 @@ pub struct BuildConfig {
     pub force_rebuild: bool,
     /// Output a build plan to stdout instead of actually compiling.
     pub build_plan: bool,
-    /// Use Cargo itself as the wrapper around rustc, only used for `cargo fix`.
-    pub cargo_as_rustc_wrapper: bool,
-    /// Extra env vars to inject into rustc commands.
-    pub extra_rustc_env: Vec<(String, String)>,
-    /// Extra args to inject into rustc commands.
-    pub extra_rustc_args: Vec<String>,
+    /// An optional wrapper, if any, used to wrap rustc invocations
+    pub rustc_wrapper: Option<ProcessBuilder>,
     pub rustfix_diagnostic_server: RefCell<Option<RustfixDiagnosticServer>>,
 }
 
@@ -65,7 +62,17 @@ impl BuildConfig {
                 failure::bail!("target was empty")
             }
         }
-        let cfg_target = config.get_string("build.target")?.map(|s| s.val);
+        let cfg_target = match config.get_string("build.target")? {
+            Some(ref target) if target.val.ends_with(".json") => {
+                let path = target.definition.root(config).join(&target.val);
+                let path_string = path
+                    .into_os_string()
+                    .into_string()
+                    .map_err(|_| failure::format_err!("Target path is not valid unicode"));
+                Some(path_string?)
+            }
+            other => other.map(|t| t.val),
+        };
         let target = requested_target.or(cfg_target);
 
         if jobs == Some(0) {
@@ -88,9 +95,7 @@ impl BuildConfig {
             message_format: MessageFormat::Human,
             force_rebuild: false,
             build_plan: false,
-            cargo_as_rustc_wrapper: false,
-            extra_rustc_env: Vec::new(),
-            extra_rustc_args: Vec::new(),
+            rustc_wrapper: None,
             rustfix_diagnostic_server: RefCell::new(None),
         })
     }
