@@ -15,40 +15,35 @@
 //! (for example, with and without tests), so we actually build a dependency
 //! graph of `Unit`s, which capture these properties.
 
-use std::cell::RefCell;
-use std::collections::{HashMap, HashSet};
-
-use log::trace;
-
-use super::{BuildContext, CompileMode, Kind, Unit};
+use crate::core::compiler::Unit;
+use crate::core::compiler::{BuildContext, CompileMode, Context, Kind};
 use crate::core::dependency::Kind as DepKind;
 use crate::core::package::Downloads;
 use crate::core::profiles::UnitFor;
 use crate::core::{Package, PackageId, Target};
 use crate::CargoResult;
+use log::trace;
+use std::collections::{HashMap, HashSet};
 
 struct State<'a: 'tmp, 'cfg: 'a, 'tmp> {
-    bcx: &'tmp BuildContext<'a, 'cfg>,
-    deps: &'tmp mut HashMap<Unit<'a>, Vec<Unit<'a>>>,
-    pkgs: RefCell<&'tmp mut HashMap<PackageId, &'a Package>>,
+    cx: &'tmp mut Context<'a, 'cfg>,
     waiting_on_download: HashSet<PackageId>,
     downloads: Downloads<'a, 'cfg>,
 }
 
 pub fn build_unit_dependencies<'a, 'cfg>(
+    cx: &mut Context<'a, 'cfg>,
     roots: &[Unit<'a>],
-    bcx: &BuildContext<'a, 'cfg>,
-    deps: &mut HashMap<Unit<'a>, Vec<Unit<'a>>>,
-    pkgs: &mut HashMap<PackageId, &'a Package>,
 ) -> CargoResult<()> {
-    assert!(deps.is_empty(), "can only build unit deps once");
+    assert!(
+        cx.unit_dependencies.is_empty(),
+        "can only build unit deps once"
+    );
 
     let mut state = State {
-        bcx,
-        deps,
-        pkgs: RefCell::new(pkgs),
+        downloads: cx.bcx.packages.enable_download()?,
+        cx,
         waiting_on_download: HashSet::new(),
-        downloads: bcx.packages.enable_download()?,
     };
 
     loop {
@@ -59,16 +54,16 @@ pub fn build_unit_dependencies<'a, 'cfg>(
             // We check the global test mode to see if we are running in `cargo
             // test` in which case we ensure all dependencies have `panic`
             // cleared, and avoid building the lib thrice (once with `panic`, once
-            // without, once for --test).  In particular, the lib included for
-            // doctests and examples are `Build` mode here.
-            let unit_for = if unit.mode.is_any_test() || bcx.build_config.test() {
+            // without, once for `--test`). In particular, the lib included for
+            // Doc tests and examples are `Build` mode here.
+            let unit_for = if unit.mode.is_any_test() || state.cx.bcx.build_config.test() {
                 UnitFor::new_test()
             } else if unit.target.is_custom_build() {
                 // This normally doesn't happen, except `clean` aggressively
                 // generates all units.
                 UnitFor::new_build()
             } else if unit.target.for_host() {
-                // proc-macro/plugin should never have panic set.
+                // Proc macro / plugin should never have panic set.
                 UnitFor::new_compiler()
             } else {
                 UnitFor::new_normal()
@@ -78,15 +73,26 @@ pub fn build_unit_dependencies<'a, 'cfg>(
 
         if !state.waiting_on_download.is_empty() {
             state.finish_some_downloads()?;
-            state.deps.clear();
+            state.cx.unit_dependencies.clear();
         } else {
             break;
         }
     }
-    trace!("ALL UNIT DEPENDENCIES {:#?}", state.deps);
 
     connect_run_custom_build_deps(&mut state);
 
+    trace!("ALL UNIT DEPENDENCIES {:#?}", state.cx.unit_dependencies);
+
+    record_units_requiring_metadata(state.cx);
+
+    // Dependencies are used in tons of places throughout the backend, many of
+    // which affect the determinism of the build itself. As a result be sure
+    // that dependency lists are always sorted to ensure we've always got a
+    // deterministic output.
+    for list in state.cx.unit_dependencies.values_mut() {
+        list.sort();
+    }
+
     Ok(())
 }
 
@@ -95,16 +101,16 @@ fn deps_of<'a, 'cfg, 'tmp>(
     state: &mut State<'a, 'cfg, 'tmp>,
     unit_for: UnitFor,
 ) -> CargoResult<()> {
-    // Currently the `deps` map does not include `unit_for`.  This should
-    // be safe for now.  `TestDependency` only exists to clear the `panic`
+    // Currently the `unit_dependencies` map does not include `unit_for`. This should
+    // be safe for now. `TestDependency` only exists to clear the `panic`
     // flag, and you'll never ask for a `unit` with `panic` set as a
-    // `TestDependency`.  `CustomBuild` should also be fine since if the
+    // `TestDependency`. `CustomBuild` should also be fine since if the
     // requested unit's settings are the same as `Any`, `CustomBuild` can't
     // affect anything else in the hierarchy.
-    if !state.deps.contains_key(unit) {
+    if !state.cx.unit_dependencies.contains_key(unit) {
         let unit_deps = compute_deps(unit, state, unit_for)?;
         let to_insert: Vec<_> = unit_deps.iter().map(|&(unit, _)| unit).collect();
-        state.deps.insert(*unit, to_insert);
+        state.cx.unit_dependencies.insert(*unit, to_insert);
         for (unit, unit_for) in unit_deps {
             deps_of(&unit, state, unit_for)?;
         }
@@ -112,9 +118,9 @@ fn deps_of<'a, 'cfg, 'tmp>(
     Ok(())
 }
 
-/// For a package, return all targets which are registered as dependencies
+/// For a package, returns all targets that are registered as dependencies
 /// for that package.
-/// This returns a vec of `(Unit, UnitFor)` pairs.  The `UnitFor`
+/// This returns a `Vec` of `(Unit, UnitFor)` pairs. The `UnitFor`
 /// is the profile type that should be used for dependencies of the unit.
 fn compute_deps<'a, 'cfg, 'tmp>(
     unit: &Unit<'a>,
@@ -122,13 +128,13 @@ fn compute_deps<'a, 'cfg, 'tmp>(
     unit_for: UnitFor,
 ) -> CargoResult<Vec<(Unit<'a>, UnitFor)>> {
     if unit.mode.is_run_custom_build() {
-        return compute_deps_custom_build(unit, state.bcx);
+        return compute_deps_custom_build(unit, state.cx.bcx);
     } else if unit.mode.is_doc() && !unit.mode.is_any_test() {
-        // Note: This does not include Doctest.
+        // Note: this does not include doc test.
         return compute_deps_doc(unit, state);
     }
 
-    let bcx = state.bcx;
+    let bcx = state.cx.bcx;
     let id = unit.pkg.package_id();
     let deps = bcx.resolve.deps(id).filter(|&(_id, deps)| {
         assert!(!deps.is_empty());
@@ -140,8 +146,8 @@ fn compute_deps<'a, 'cfg, 'tmp>(
                 return false;
             }
 
-            // If this dependency is *not* a transitive dependency, then it
-            // only applies to test/example targets
+            // If this dependency is **not** a transitive dependency, then it
+            // only applies to test/example targets.
             if !dep.is_transitive()
                 && !unit.target.is_test()
                 && !unit.target.is_example()
@@ -180,8 +186,19 @@ fn compute_deps<'a, 'cfg, 'tmp>(
         };
         let mode = check_or_build_mode(unit.mode, lib);
         let dep_unit_for = unit_for.with_for_host(lib.for_host());
-        let unit = new_unit(bcx, pkg, lib, dep_unit_for, unit.kind.for_target(lib), mode);
-        ret.push((unit, dep_unit_for));
+
+        if bcx.config.cli_unstable().dual_proc_macros
+            && lib.proc_macro()
+            && unit.kind == Kind::Target
+        {
+            let unit = new_unit(bcx, pkg, lib, dep_unit_for, Kind::Target, mode);
+            ret.push((unit, dep_unit_for));
+            let unit = new_unit(bcx, pkg, lib, dep_unit_for, Kind::Host, mode);
+            ret.push((unit, dep_unit_for));
+        } else {
+            let unit = new_unit(bcx, pkg, lib, dep_unit_for, unit.kind.for_target(lib), mode);
+            ret.push((unit, dep_unit_for));
+        }
     }
 
     // If this target is a build script, then what we've collected so far is
@@ -249,7 +266,7 @@ fn compute_deps_custom_build<'a, 'cfg>(
 ) -> CargoResult<Vec<(Unit<'a>, UnitFor)>> {
     // When not overridden, then the dependencies to run a build script are:
     //
-    // 1. Compiling the build script itself
+    // 1. Compiling the build script itself.
     // 2. For each immediate dependency of our package which has a `links`
     //    key, the execution of that build script.
     //
@@ -261,7 +278,8 @@ fn compute_deps_custom_build<'a, 'cfg>(
         unit.pkg,
         unit.target,
         UnitFor::new_build(),
-        Kind::Host, // build scripts always compiled for the host
+        // Build scripts always compiled for the host.
+        Kind::Host,
         CompileMode::Build,
     );
     // All dependencies of this unit should use profiles for custom
@@ -269,12 +287,12 @@ fn compute_deps_custom_build<'a, 'cfg>(
     Ok(vec![(unit, UnitFor::new_build())])
 }
 
-/// Returns the dependencies necessary to document a package
+/// Returns the dependencies necessary to document a package.
 fn compute_deps_doc<'a, 'cfg, 'tmp>(
     unit: &Unit<'a>,
     state: &mut State<'a, 'cfg, 'tmp>,
 ) -> CargoResult<Vec<(Unit<'a>, UnitFor)>> {
-    let bcx = state.bcx;
+    let bcx = state.cx.bcx;
     let deps = bcx
         .resolve
         .deps(unit.pkg.package_id())
@@ -298,8 +316,8 @@ fn compute_deps_doc<'a, 'cfg, 'tmp>(
             Some(lib) => lib,
             None => continue,
         };
-        // rustdoc only needs rmeta files for regular dependencies.
-        // However, for plugins/proc-macros, deps should be built like normal.
+        // Rustdoc only needs rmeta files for regular dependencies.
+        // However, for plugins/proc macros, deps should be built like normal.
         let mode = check_or_build_mode(unit.mode, lib);
         let dep_unit_for = UnitFor::new_normal().with_for_host(lib.for_host());
         let lib_unit = new_unit(bcx, dep, lib, dep_unit_for, unit.kind.for_target(lib), mode);
@@ -318,10 +336,10 @@ fn compute_deps_doc<'a, 'cfg, 'tmp>(
         }
     }
 
-    // Be sure to build/run the build script for documented libraries as
+    // Be sure to build/run the build script for documented libraries.
     ret.extend(dep_build_script(unit, bcx));
 
-    // If we document a binary, we need the library available
+    // If we document a binary, we need the library available.
     if unit.target.is_bin() {
         ret.extend(maybe_lib(unit, bcx, UnitFor::new_normal()));
     }
@@ -330,7 +348,7 @@ fn compute_deps_doc<'a, 'cfg, 'tmp>(
 
 fn maybe_lib<'a>(
     unit: &Unit<'a>,
-    bcx: &BuildContext<'_, '_>,
+    bcx: &BuildContext<'a, '_>,
     unit_for: UnitFor,
 ) -> Option<(Unit<'a>, UnitFor)> {
     unit.pkg.targets().iter().find(|t| t.linkable()).map(|t| {
@@ -349,7 +367,7 @@ fn maybe_lib<'a>(
 /// build script.
 fn dep_build_script<'a>(
     unit: &Unit<'a>,
-    bcx: &BuildContext<'_, '_>,
+    bcx: &BuildContext<'a, '_>,
 ) -> Option<(Unit<'a>, UnitFor)> {
     unit.pkg
         .targets()
@@ -358,16 +376,15 @@ fn dep_build_script<'a>(
         .map(|t| {
             // The profile stored in the Unit is the profile for the thing
             // the custom build script is running for.
-            (
-                Unit {
-                    pkg: unit.pkg,
-                    target: t,
-                    profile: bcx.profiles.get_profile_run_custom_build(&unit.profile),
-                    kind: unit.kind,
-                    mode: CompileMode::RunCustomBuild,
-                },
-                UnitFor::new_build(),
-            )
+            let unit = bcx.units.intern(
+                unit.pkg,
+                t,
+                bcx.profiles.get_profile_run_custom_build(&unit.profile),
+                unit.kind,
+                CompileMode::RunCustomBuild,
+            );
+
+            (unit, UnitFor::new_build())
         })
 }
 
@@ -376,7 +393,7 @@ fn check_or_build_mode(mode: CompileMode, target: &Target) -> CompileMode {
     match mode {
         CompileMode::Check { .. } | CompileMode::Doc { .. } => {
             if target.for_host() {
-                // Plugin and proc-macro targets should be compiled like
+                // Plugin and proc macro targets should be compiled like
                 // normal.
                 CompileMode::Build
             } else {
@@ -390,7 +407,7 @@ fn check_or_build_mode(mode: CompileMode, target: &Target) -> CompileMode {
 }
 
 fn new_unit<'a>(
-    bcx: &BuildContext<'_, '_>,
+    bcx: &BuildContext<'a, '_>,
     pkg: &'a Package,
     target: &'a Target,
     unit_for: UnitFor,
@@ -404,13 +421,8 @@ fn new_unit<'a>(
         mode,
         bcx.build_config.release,
     );
-    Unit {
-        pkg,
-        target,
-        profile,
-        kind,
-        mode,
-    }
+
+    bcx.units.intern(pkg, target, profile, kind, mode)
 }
 
 /// Fill in missing dependencies for units of the `RunCustomBuild`
@@ -433,7 +445,7 @@ fn connect_run_custom_build_deps(state: &mut State<'_, '_, '_>) {
         // have the build script as the key and the library would be in the
         // value's set.
         let mut reverse_deps = HashMap::new();
-        for (unit, deps) in state.deps.iter() {
+        for (unit, deps) in state.cx.unit_dependencies.iter() {
             for dep in deps {
                 if dep.mode == CompileMode::RunCustomBuild {
                     reverse_deps
@@ -444,7 +456,7 @@ fn connect_run_custom_build_deps(state: &mut State<'_, '_, '_>) {
             }
         }
 
-        // And next we take a look at all build scripts executions listed in the
+        // Next, we take a look at all build scripts executions listed in the
         // dependency map. Our job here is to take everything that depends on
         // this build script (from our reverse map above) and look at the other
         // package dependencies of these parents.
@@ -454,7 +466,8 @@ fn connect_run_custom_build_deps(state: &mut State<'_, '_, '_>) {
         // `dep_build_script` to manufacture an appropriate build script unit to
         // depend on.
         for unit in state
-            .deps
+            .cx
+            .unit_dependencies
             .keys()
             .filter(|k| k.mode == CompileMode::RunCustomBuild)
         {
@@ -465,13 +478,13 @@ fn connect_run_custom_build_deps(state: &mut State<'_, '_, '_>) {
 
             let to_add = reverse_deps
                 .iter()
-                .flat_map(|reverse_dep| state.deps[reverse_dep].iter())
+                .flat_map(|reverse_dep| state.cx.unit_dependencies[reverse_dep].iter())
                 .filter(|other| {
                     other.pkg != unit.pkg
                         && other.target.linkable()
                         && other.pkg.manifest().links().is_some()
                 })
-                .filter_map(|other| dep_build_script(other, state.bcx).map(|p| p.0))
+                .filter_map(|other| dep_build_script(other, state.cx.bcx).map(|p| p.0))
                 .collect::<HashSet<_>>();
 
             if !to_add.is_empty() {
@@ -482,21 +495,39 @@ fn connect_run_custom_build_deps(state: &mut State<'_, '_, '_>) {
 
     // And finally, add in all the missing dependencies!
     for (unit, new_deps) in new_deps {
-        state.deps.get_mut(&unit).unwrap().extend(new_deps);
+        state
+            .cx
+            .unit_dependencies
+            .get_mut(&unit)
+            .unwrap()
+            .extend(new_deps);
+    }
+}
+
+/// Records the list of units which are required to emit metadata.
+///
+/// Units which depend only on the metadata of others requires the others to
+/// actually produce metadata, so we'll record that here.
+fn record_units_requiring_metadata(cx: &mut Context<'_, '_>) {
+    for (key, deps) in cx.unit_dependencies.iter() {
+        for dep in deps {
+            if cx.only_requires_rmeta(key, dep) {
+                cx.rmeta_required.insert(*dep);
+            }
+        }
     }
 }
 
 impl<'a, 'cfg, 'tmp> State<'a, 'cfg, 'tmp> {
     fn get(&mut self, id: PackageId) -> CargoResult<Option<&'a Package>> {
-        let mut pkgs = self.pkgs.borrow_mut();
-        if let Some(pkg) = pkgs.get(&id) {
+        if let Some(pkg) = self.cx.package_cache.get(&id) {
             return Ok(Some(pkg));
         }
         if !self.waiting_on_download.insert(id) {
             return Ok(None);
         }
         if let Some(pkg) = self.downloads.start(id)? {
-            pkgs.insert(id, pkg);
+            self.cx.package_cache.insert(id, pkg);
             self.waiting_on_download.remove(&id);
             return Ok(Some(pkg));
         }
@@ -516,7 +547,7 @@ impl<'a, 'cfg, 'tmp> State<'a, 'cfg, 'tmp> {
         loop {
             let pkg = self.downloads.wait()?;
             self.waiting_on_download.remove(&pkg.package_id());
-            self.pkgs.borrow_mut().insert(pkg.package_id(), pkg);
+            self.cx.package_cache.insert(pkg.package_id(), pkg);
 
             // Arbitrarily choose that 5 or more packages concurrently download
             // is a good enough number to "fill the network pipe". If we have
