@@ -1,38 +1,37 @@
-//!
-//! Cargo compile currently does the following steps:
+//! Cargo `compile` currently does the following steps.
 //!
 //! All configurations are already injected as environment variables via the
-//! main cargo command
+//! main cargo command.
 //!
-//! 1. Read the manifest
+//! 1. Read the manifest.
 //! 2. Shell out to `cargo-resolve` with a list of dependencies and sources as
-//!    stdin
+//!    stdin.
 //!
-//!    a. Shell out to `--do update` and `--do list` for each source
-//!    b. Resolve dependencies and return a list of name/version/source
+//!    a. Shell out to `--do update` and `--do list` for each source.
+//!    b. Resolve dependencies and return a list of name/version/source.
 //!
-//! 3. Shell out to `--do download` for each source
+//! 3. Shell out to `--do download` for each source.
 //! 4. Shell out to `--do get` for each source, and build up the list of paths
-//!    to pass to rustc -L
+//!    to pass to `rustc -L`.
 //! 5. Call `cargo-rustc` with the results of the resolver zipped together with
-//!    the results of the `get`
+//!    the results of the `get`.
 //!
-//!    a. Topologically sort the dependencies
+//!    a. Topologically sort the dependencies.
 //!    b. Compile each dependency in order, passing in the -L's pointing at each
-//!       previously compiled dependency
-//!
+//!       previously compiled dependency.
 
-use std::collections::{HashMap, HashSet};
+use std::collections::{BTreeSet, HashMap, HashSet};
+use std::iter::FromIterator;
 use std::path::PathBuf;
+use std::rc::Rc;
 use std::sync::Arc;
 
-use crate::core::compiler::{
-    BuildConfig, BuildContext, Compilation, Context, DefaultExecutor, Executor,
-};
+use crate::core::compiler::{BuildConfig, BuildContext, Compilation, Context};
 use crate::core::compiler::{CompileMode, Kind, Unit};
+use crate::core::compiler::{DefaultExecutor, Executor, UnitInterner};
 use crate::core::profiles::{Profiles, UnitFor};
 use crate::core::resolver::{Method, Resolve};
-use crate::core::{Package, Source, Target};
+use crate::core::{Package, Target};
 use crate::core::{PackageId, PackageIdSpec, TargetKind, Workspace};
 use crate::ops;
 use crate::util::config::Config;
@@ -110,22 +109,37 @@ impl Packages {
     }
 
     pub fn to_package_id_specs(&self, ws: &Workspace<'_>) -> CargoResult<Vec<PackageIdSpec>> {
-        let specs = match *self {
+        let specs = match self {
             Packages::All => ws
                 .members()
                 .map(Package::package_id)
                 .map(PackageIdSpec::from_package_id)
                 .collect(),
-            Packages::OptOut(ref opt_out) => ws
-                .members()
-                .map(Package::package_id)
-                .map(PackageIdSpec::from_package_id)
-                .filter(|p| opt_out.iter().position(|x| *x == p.name()).is_none())
-                .collect(),
-            Packages::Packages(ref packages) if packages.is_empty() => {
+            Packages::OptOut(opt_out) => {
+                let mut opt_out = BTreeSet::from_iter(opt_out.iter().cloned());
+                let packages = ws
+                    .members()
+                    .filter(|pkg| !opt_out.remove(pkg.name().as_str()))
+                    .map(Package::package_id)
+                    .map(PackageIdSpec::from_package_id)
+                    .collect();
+                if !opt_out.is_empty() {
+                    ws.config().shell().warn(format!(
+                        "excluded package(s) {} not found in workspace `{}`",
+                        opt_out
+                            .iter()
+                            .map(|x| x.as_ref())
+                            .collect::<Vec<_>>()
+                            .join(", "),
+                        ws.root().display(),
+                    ))?;
+                }
+                packages
+            }
+            Packages::Packages(packages) if packages.is_empty() => {
                 vec![PackageIdSpec::from_package_id(ws.current()?.package_id())]
             }
-            Packages::Packages(ref packages) => packages
+            Packages::Packages(packages) => packages
                 .iter()
                 .map(|p| PackageIdSpec::parse(p))
                 .collect::<CargoResult<Vec<_>>>()?,
@@ -152,11 +166,11 @@ impl Packages {
         let packages: Vec<_> = match self {
             Packages::Default => ws.default_members().collect(),
             Packages::All => ws.members().collect(),
-            Packages::OptOut(ref opt_out) => ws
+            Packages::OptOut(opt_out) => ws
                 .members()
                 .filter(|pkg| !opt_out.iter().any(|name| pkg.name().as_str() == name))
                 .collect(),
-            Packages::Packages(ref pkgs) => pkgs
+            Packages::Packages(packages) => packages
                 .iter()
                 .map(|name| {
                     ws.members()
@@ -172,6 +186,27 @@ impl Packages {
         };
         Ok(packages)
     }
+
+    /// Returns whether or not the user needs to pass a `-p` flag to target a
+    /// specific package in the workspace.
+    pub fn needs_spec_flag(&self, ws: &Workspace<'_>) -> bool {
+        match self {
+            Packages::Default => ws.default_members().count() > 1,
+            Packages::All => ws.members().count() > 1,
+            Packages::Packages(_) => true,
+            Packages::OptOut(_) => true,
+        }
+    }
+}
+
+#[derive(Debug, PartialEq, Eq)]
+pub enum LibRule {
+    /// Include the library, fail if not present
+    True,
+    /// Include the library if present
+    Default,
+    /// Exclude the library
+    False,
 }
 
 #[derive(Debug)]
@@ -188,7 +223,7 @@ pub enum CompileFilter {
     },
     Only {
         all_targets: bool,
-        lib: bool,
+        lib: LibRule,
         bins: FilterRule,
         examples: FilterRule,
         tests: FilterRule,
@@ -212,12 +247,11 @@ pub fn compile_with_exec<'a>(
     exec: &Arc<dyn Executor>,
 ) -> CargoResult<Compilation<'a>> {
     ws.emit_warnings()?;
-    compile_ws(ws, None, options, exec)
+    compile_ws(ws, options, exec)
 }
 
 pub fn compile_ws<'a>(
     ws: &Workspace<'a>,
-    source: Option<Box<dyn Source + 'a>>,
     options: &CompileOptions<'a>,
     exec: &Arc<dyn Executor>,
 ) -> CargoResult<Compilation<'a>> {
@@ -235,6 +269,27 @@ pub fn compile_ws<'a>(
         ref export_dir,
     } = *options;
 
+    match build_config.mode {
+        CompileMode::Test
+        | CompileMode::Build
+        | CompileMode::Check { .. }
+        | CompileMode::Bench
+        | CompileMode::RunCustomBuild => {
+            if std::env::var("RUST_FLAGS").is_ok() {
+                config.shell().warn(
+                    "Cargo does not read `RUST_FLAGS` environment variable. Did you mean `RUSTFLAGS`?",
+                )?;
+            }
+        }
+        CompileMode::Doc { .. } | CompileMode::Doctest => {
+            if std::env::var("RUSTDOC_FLAGS").is_ok() {
+                config.shell().warn(
+                    "Cargo does not read `RUSTDOC_FLAGS` environment variable. Did you mean `RUSTDOCFLAGS`?"
+                )?;
+            }
+        }
+    }
+
     let default_arch_kind = if build_config.requested_target.is_some() {
         Kind::Target
     } else {
@@ -245,11 +300,11 @@ pub fn compile_ws<'a>(
     let features = Method::split_features(features);
     let method = Method::Required {
         dev_deps: ws.require_optional_deps() || filter.need_dev_deps(build_config.mode),
-        features: &features,
+        features: Rc::new(features),
         all_features,
         uses_default_features: !no_default_features,
     };
-    let resolve = ops::resolve_ws_with_method(ws, source, method, &specs)?;
+    let resolve = ops::resolve_ws_with_method(ws, method, &specs)?;
     let (packages, resolve_with_overrides) = resolve;
 
     let to_build_ids = specs
@@ -294,6 +349,17 @@ pub fn compile_ws<'a>(
     let profiles = ws.profiles();
     profiles.validate_packages(&mut config.shell(), &packages)?;
 
+    let interner = UnitInterner::new();
+    let mut bcx = BuildContext::new(
+        ws,
+        &resolve_with_overrides,
+        &packages,
+        config,
+        build_config,
+        profiles,
+        &interner,
+        HashMap::new(),
+    )?;
     let units = generate_targets(
         ws,
         profiles,
@@ -301,42 +367,32 @@ pub fn compile_ws<'a>(
         filter,
         default_arch_kind,
         &resolve_with_overrides,
-        build_config,
+        &bcx,
     )?;
 
-    let mut extra_compiler_args = HashMap::new();
     if let Some(args) = extra_args {
         if units.len() != 1 {
             failure::bail!(
                 "extra arguments to `{}` can only be passed to one \
-                 target, consider filtering\nthe package by passing \
-                 e.g. `--lib` or `--bin NAME` to specify a single target",
+                 target, consider filtering\nthe package by passing, \
+                 e.g., `--lib` or `--bin NAME` to specify a single target",
                 extra_args_name
             );
         }
-        extra_compiler_args.insert(units[0], args);
+        bcx.extra_compiler_args.insert(units[0], args);
     }
     if let Some(args) = local_rustdoc_args {
         for unit in &units {
             if unit.mode.is_doc() {
-                extra_compiler_args.insert(*unit, args.clone());
+                bcx.extra_compiler_args.insert(*unit, args.clone());
             }
         }
     }
 
     let ret = {
         let _p = profile::start("compiling");
-        let bcx = BuildContext::new(
-            ws,
-            &resolve_with_overrides,
-            &packages,
-            config,
-            &build_config,
-            profiles,
-            extra_compiler_args,
-        )?;
         let cx = Context::new(config, &bcx)?;
-        cx.compile(&units, export_dir.clone(), &exec)?
+        cx.compile(&units, export_dir.clone(), exec)?
     };
 
     Ok(ret)
@@ -351,6 +407,10 @@ impl FilterRule {
         }
     }
 
+    pub fn none() -> FilterRule {
+        FilterRule::Just(Vec::new())
+    }
+
     fn matches(&self, target: &Target) -> bool {
         match *self {
             FilterRule::All => true,
@@ -374,7 +434,8 @@ impl FilterRule {
 }
 
 impl CompileFilter {
-    pub fn new(
+    /// Construct a CompileFilter from raw command line arguments.
+    pub fn from_raw_arguments(
         lib_only: bool,
         bins: Vec<String>,
         all_bins: bool,
@@ -386,6 +447,11 @@ impl CompileFilter {
         all_bens: bool,
         all_targets: bool,
     ) -> CompileFilter {
+        let rule_lib = if lib_only {
+            LibRule::True
+        } else {
+            LibRule::False
+        };
         let rule_bins = FilterRule::new(bins, all_bins);
         let rule_tsts = FilterRule::new(tsts, all_tsts);
         let rule_exms = FilterRule::new(exms, all_exms);
@@ -394,13 +460,26 @@ impl CompileFilter {
         if all_targets {
             CompileFilter::Only {
                 all_targets: true,
-                lib: true,
+                lib: LibRule::Default,
                 bins: FilterRule::All,
                 examples: FilterRule::All,
                 benches: FilterRule::All,
                 tests: FilterRule::All,
             }
-        } else if lib_only
+        } else {
+            CompileFilter::new(rule_lib, rule_bins, rule_tsts, rule_exms, rule_bens)
+        }
+    }
+
+    /// Construct a CompileFilter from underlying primitives.
+    pub fn new(
+        rule_lib: LibRule,
+        rule_bins: FilterRule,
+        rule_tsts: FilterRule,
+        rule_exms: FilterRule,
+        rule_bens: FilterRule,
+    ) -> CompileFilter {
+        if rule_lib == LibRule::True
             || rule_bins.is_specific()
             || rule_tsts.is_specific()
             || rule_exms.is_specific()
@@ -408,7 +487,7 @@ impl CompileFilter {
         {
             CompileFilter::Only {
                 all_targets: false,
-                lib: lib_only,
+                lib: rule_lib,
                 bins: rule_bins,
                 examples: rule_exms,
                 benches: rule_bens,
@@ -444,7 +523,7 @@ impl CompileFilter {
         match *self {
             CompileFilter::Default { .. } => true,
             CompileFilter::Only {
-                lib,
+                ref lib,
                 ref bins,
                 ref examples,
                 ref tests,
@@ -456,7 +535,13 @@ impl CompileFilter {
                     TargetKind::Test => tests,
                     TargetKind::Bench => benches,
                     TargetKind::ExampleBin | TargetKind::ExampleLib(..) => examples,
-                    TargetKind::Lib(..) => return lib,
+                    TargetKind::Lib(..) => {
+                        return match *lib {
+                            LibRule::True => true,
+                            LibRule::Default => true,
+                            LibRule::False => false,
+                        };
+                    }
                     TargetKind::CustomBuild => return false,
                 };
                 rule.matches(target)
@@ -474,7 +559,7 @@ impl CompileFilter {
 
 /// A proposed target.
 ///
-/// Proposed targets are later filtered into actual Units based on whether or
+/// Proposed targets are later filtered into actual `Unit`s based on whether or
 /// not the target requires its features to be present.
 #[derive(Debug)]
 struct Proposal<'a> {
@@ -489,8 +574,7 @@ struct Proposal<'a> {
 }
 
 /// Generates all the base targets for the packages the user has requested to
-/// compile. Dependencies for these targets are computed later in
-/// `unit_dependencies`.
+/// compile. Dependencies for these targets are computed later in `unit_dependencies`.
 fn generate_targets<'a>(
     ws: &Workspace<'_>,
     profiles: &Profiles,
@@ -498,32 +582,32 @@ fn generate_targets<'a>(
     filter: &CompileFilter,
     default_arch_kind: Kind,
     resolve: &Resolve,
-    build_config: &BuildConfig,
+    bcx: &BuildContext<'a, '_>,
 ) -> CargoResult<Vec<Unit<'a>>> {
-    // Helper for creating a Unit struct.
+    // Helper for creating a `Unit` struct.
     let new_unit = |pkg: &'a Package, target: &'a Target, target_mode: CompileMode| {
-        let unit_for = if build_config.mode.is_any_test() {
-            // NOTE: The UnitFor here is subtle.  If you have a profile
+        let unit_for = if bcx.build_config.mode.is_any_test() {
+            // NOTE: the `UnitFor` here is subtle. If you have a profile
             // with `panic` set, the `panic` flag is cleared for
-            // tests/benchmarks and their dependencies.  If this
+            // tests/benchmarks and their dependencies. If this
             // was `normal`, then the lib would get compiled three
             // times (once with panic, once without, and once with
-            // --test).
+            // `--test`).
             //
-            // This would cause a problem for Doc tests, which would fail
+            // This would cause a problem for doc tests, which would fail
             // because `rustdoc` would attempt to link with both libraries
             // at the same time. Also, it's probably not important (or
             // even desirable?) for rustdoc to link with a lib with
             // `panic` set.
             //
             // As a consequence, Examples and Binaries get compiled
-            // without `panic` set.  This probably isn't a bad deal.
+            // without `panic` set. This probably isn't a bad deal.
             //
             // Forcing the lib to be compiled three times during `cargo
             // test` is probably also not desirable.
             UnitFor::new_test()
         } else if target.for_host() {
-            // proc-macro/plugin should not have `panic` set.
+            // Proc macro / plugin should not have `panic` set.
             UnitFor::new_compiler()
         } else {
             UnitFor::new_normal()
@@ -545,18 +629,18 @@ fn generate_targets<'a>(
                 TargetKind::Bench => CompileMode::Bench,
                 _ => CompileMode::Build,
             },
-            // CompileMode::Bench is only used to inform filter_default_targets
+            // `CompileMode::Bench` is only used to inform `filter_default_targets`
             // which command is being used (`cargo bench`). Afterwards, tests
             // and benches are treated identically. Switching the mode allows
-            // de-duplication of units that are essentially identical.  For
+            // de-duplication of units that are essentially identical. For
             // example, `cargo build --all-targets --release` creates the units
             // (lib profile:bench, mode:test) and (lib profile:bench, mode:bench)
-            // and since these are the same, we want them to be de-duped in
+            // and since these are the same, we want them to be de-duplicated in
             // `unit_dependencies`.
             CompileMode::Bench => CompileMode::Test,
             _ => target_mode,
         };
-        // Plugins or proc-macro should be built for the host.
+        // Plugins or proc macros should be built for the host.
         let kind = if target.for_host() {
             Kind::Host
         } else {
@@ -567,15 +651,9 @@ fn generate_targets<'a>(
             ws.is_member(pkg),
             unit_for,
             target_mode,
-            build_config.release,
+            bcx.build_config.release,
         );
-        Unit {
-            pkg,
-            target,
-            profile,
-            kind,
-            mode: target_mode,
-        }
+        bcx.units.intern(pkg, target, profile, kind, target_mode)
     };
 
     // Create a list of proposed targets.
@@ -586,14 +664,14 @@ fn generate_targets<'a>(
             required_features_filterable,
         } => {
             for pkg in packages {
-                let default = filter_default_targets(pkg.targets(), build_config.mode);
+                let default = filter_default_targets(pkg.targets(), bcx.build_config.mode);
                 proposals.extend(default.into_iter().map(|target| Proposal {
                     pkg,
                     target,
                     requires_features: !required_features_filterable,
-                    mode: build_config.mode,
+                    mode: bcx.build_config.mode,
                 }));
-                if build_config.mode == CompileMode::Test {
+                if bcx.build_config.mode == CompileMode::Test {
                     if let Some(t) = pkg
                         .targets()
                         .iter()
@@ -611,35 +689,29 @@ fn generate_targets<'a>(
         }
         CompileFilter::Only {
             all_targets,
-            lib,
+            ref lib,
             ref bins,
             ref examples,
             ref tests,
             ref benches,
         } => {
-            if lib {
+            if *lib != LibRule::False {
                 let mut libs = Vec::new();
-                for pkg in packages {
-                    for target in pkg.targets().iter().filter(|t| t.is_lib()) {
-                        if build_config.mode == CompileMode::Doctest && !target.doctestable() {
-                            ws.config()
-                                .shell()
-                                .warn(format!(
-                                "doc tests are not supported for crate type(s) `{}` in package `{}`",
-                                target.rustc_crate_types().join(", "),
-                                pkg.name()
-                            ))?;
-                        } else {
-                            libs.push(Proposal {
-                                pkg,
-                                target,
-                                requires_features: false,
-                                mode: build_config.mode,
-                            });
-                        }
+                for proposal in
+                    filter_targets(packages, Target::is_lib, false, bcx.build_config.mode)
+                {
+                    let Proposal { target, pkg, .. } = proposal;
+                    if bcx.build_config.mode == CompileMode::Doctest && !target.doctestable() {
+                        ws.config().shell().warn(format!(
+                            "doc tests are not supported for crate type(s) `{}` in package `{}`",
+                            target.rustc_crate_types().join(", "),
+                            pkg.name()
+                        ))?;
+                    } else {
+                        libs.push(proposal)
                     }
                 }
-                if !all_targets && libs.is_empty() {
+                if !all_targets && libs.is_empty() && *lib == LibRule::True {
                     let names = packages.iter().map(|pkg| pkg.name()).collect::<Vec<_>>();
                     if names.len() == 1 {
                         failure::bail!("no library targets found in package `{}`", names[0]);
@@ -653,27 +725,27 @@ fn generate_targets<'a>(
                 proposals.extend(libs);
             }
 
-            // If --tests was specified, add all targets that would be
+            // If `--tests` was specified, add all targets that would be
             // generated by `cargo test`.
-            let test_filter = match *tests {
+            let test_filter = match tests {
                 FilterRule::All => Target::tested,
                 FilterRule::Just(_) => Target::is_test,
             };
-            let test_mode = match build_config.mode {
+            let test_mode = match bcx.build_config.mode {
                 CompileMode::Build => CompileMode::Test,
                 CompileMode::Check { .. } => CompileMode::Check { test: true },
-                _ => build_config.mode,
+                _ => bcx.build_config.mode,
             };
-            // If --benches was specified, add all targets that would be
+            // If `--benches` was specified, add all targets that would be
             // generated by `cargo bench`.
-            let bench_filter = match *benches {
+            let bench_filter = match benches {
                 FilterRule::All => Target::benched,
                 FilterRule::Just(_) => Target::is_bench,
             };
-            let bench_mode = match build_config.mode {
+            let bench_mode = match bcx.build_config.mode {
                 CompileMode::Build => CompileMode::Bench,
                 CompileMode::Check { .. } => CompileMode::Check { test: true },
-                _ => build_config.mode,
+                _ => bcx.build_config.mode,
             };
 
             proposals.extend(list_rule_targets(
@@ -681,14 +753,14 @@ fn generate_targets<'a>(
                 bins,
                 "bin",
                 Target::is_bin,
-                build_config.mode,
+                bcx.build_config.mode,
             )?);
             proposals.extend(list_rule_targets(
                 packages,
                 examples,
                 "example",
                 Target::is_example,
-                build_config.mode,
+                bcx.build_config.mode,
             )?);
             proposals.extend(list_rule_targets(
                 packages,
@@ -738,7 +810,7 @@ fn generate_targets<'a>(
                 .collect();
             failure::bail!(
                 "target `{}` in package `{}` requires the features: {}\n\
-                 Consider enabling them by passing e.g. `--features=\"{}\"`",
+                 Consider enabling them by passing, e.g., `--features=\"{}\"`",
                 target.name(),
                 pkg.name(),
                 quoted_required_features.join(", "),
@@ -795,10 +867,7 @@ fn filter_default_targets(targets: &[Target], mode: CompileMode) -> Vec<&Target>
     }
 }
 
-/// Returns a list of targets based on command-line target selection flags.
-/// The return value is a list of `(Package, Target, bool, CompileMode)`
-/// tuples.  The `bool` value indicates whether or not all required features
-/// *must* be present.
+/// Returns a list of proposed targets based on command-line target selection flags.
 fn list_rule_targets<'a>(
     packages: &[&'a Package],
     rule: &FilterRule,
@@ -806,25 +875,14 @@ fn list_rule_targets<'a>(
     is_expected_kind: fn(&Target) -> bool,
     mode: CompileMode,
 ) -> CargoResult<Vec<Proposal<'a>>> {
-    let mut result = Vec::new();
-    match *rule {
+    let mut proposals = Vec::new();
+    match rule {
         FilterRule::All => {
-            for pkg in packages {
-                for target in pkg.targets() {
-                    if is_expected_kind(target) {
-                        result.push(Proposal {
-                            pkg,
-                            target,
-                            requires_features: false,
-                            mode,
-                        });
-                    }
-                }
-            }
+            proposals.extend(filter_targets(packages, is_expected_kind, false, mode))
         }
-        FilterRule::Just(ref names) => {
+        FilterRule::Just(names) => {
             for name in names {
-                result.extend(find_named_targets(
+                proposals.extend(find_named_targets(
                     packages,
                     name,
                     target_desc,
@@ -834,10 +892,10 @@ fn list_rule_targets<'a>(
             }
         }
     }
-    Ok(result)
+    Ok(proposals)
 }
 
-/// Find the targets for a specifically named target.
+/// Finds the targets for a specifically named target.
 fn find_named_targets<'a>(
     packages: &[&'a Package],
     target_name: &str,
@@ -845,20 +903,9 @@ fn find_named_targets<'a>(
     is_expected_kind: fn(&Target) -> bool,
     mode: CompileMode,
 ) -> CargoResult<Vec<Proposal<'a>>> {
-    let mut result = Vec::new();
-    for pkg in packages {
-        for target in pkg.targets() {
-            if target.name() == target_name && is_expected_kind(target) {
-                result.push(Proposal {
-                    pkg,
-                    target,
-                    requires_features: true,
-                    mode,
-                });
-            }
-        }
-    }
-    if result.is_empty() {
+    let filter = |t: &Target| t.name() == target_name && is_expected_kind(t);
+    let proposals = filter_targets(packages, filter, true, mode);
+    if proposals.is_empty() {
         let suggestion = packages
             .iter()
             .flat_map(|pkg| {
@@ -880,5 +927,25 @@ fn find_named_targets<'a>(
             None => failure::bail!("no {} target named `{}`", target_desc, target_name),
         }
     }
-    Ok(result)
+    Ok(proposals)
+}
+
+fn filter_targets<'a>(
+    packages: &[&'a Package],
+    predicate: impl Fn(&Target) -> bool,
+    requires_features: bool,
+    mode: CompileMode,
+) -> Vec<Proposal<'a>> {
+    let mut proposals = Vec::new();
+    for pkg in packages {
+        for target in pkg.targets().iter().filter(|t| predicate(t)) {
+            proposals.push(Proposal {
+                pkg,
+                target,
+                requires_features,
+                mode,
+            });
+        }
+    }
+    proposals
 }
