@@ -1,5 +1,6 @@
-use std::collections::btree_map::Entry;
-use std::{env, fs};
+use failure::bail;
+use std::collections::BTreeSet;
+use std::env;
 
 use crate::core::PackageId;
 use crate::core::{PackageIdSpec, SourceId};
@@ -7,7 +8,7 @@ use crate::ops::common_for_install_and_uninstall::*;
 use crate::util::errors::CargoResult;
 use crate::util::paths;
 use crate::util::Config;
-use crate::util::{FileLock, Filesystem};
+use crate::util::Filesystem;
 
 pub fn uninstall(
     root: Option<&str>,
@@ -16,7 +17,7 @@ pub fn uninstall(
     config: &Config,
 ) -> CargoResult<()> {
     if specs.len() > 1 && !bins.is_empty() {
-        failure::bail!("A binary can only be associated with a single installed package, specifying multiple specs with --bin is redundant.");
+        bail!("A binary can only be associated with a single installed package, specifying multiple specs with --bin is redundant.");
     }
 
     let root = resolve_root(root, config)?;
@@ -62,7 +63,7 @@ pub fn uninstall(
     };
 
     if scheduled_error {
-        failure::bail!("some packages failed to uninstall");
+        bail!("some packages failed to uninstall");
     }
 
     Ok(())
@@ -74,80 +75,74 @@ pub fn uninstall_one(
     bins: &[String],
     config: &Config,
 ) -> CargoResult<()> {
-    let crate_metadata = metadata(config, root)?;
-    let metadata = read_crate_list(&crate_metadata)?;
-    let pkgid = PackageIdSpec::query_str(spec, metadata.v1().keys().cloned())?;
-    uninstall_pkgid(&crate_metadata, metadata, pkgid, bins, config)
+    let tracker = InstallTracker::load(config, root)?;
+    let all_pkgs = tracker.all_installed_bins().map(|(pkg_id, _set)| *pkg_id);
+    let pkgid = PackageIdSpec::query_str(spec, all_pkgs)?;
+    uninstall_pkgid(root, tracker, pkgid, bins, config)
 }
 
 fn uninstall_cwd(root: &Filesystem, bins: &[String], config: &Config) -> CargoResult<()> {
-    let crate_metadata = metadata(config, root)?;
-    let metadata = read_crate_list(&crate_metadata)?;
+    let tracker = InstallTracker::load(config, root)?;
     let source_id = SourceId::for_path(config.cwd())?;
     let src = path_source(source_id, config)?;
-    let (pkg, _source) = select_pkg(src, None, None, config, true, &mut |path| {
+    let pkg = select_pkg(src, None, None, config, true, &mut |path| {
         path.read_packages()
     })?;
     let pkgid = pkg.package_id();
-    uninstall_pkgid(&crate_metadata, metadata, pkgid, bins, config)
+    uninstall_pkgid(root, tracker, pkgid, bins, config)
 }
 
 fn uninstall_pkgid(
-    crate_metadata: &FileLock,
-    mut metadata: CrateListingV1,
+    root: &Filesystem,
+    mut tracker: InstallTracker,
     pkgid: PackageId,
     bins: &[String],
     config: &Config,
 ) -> CargoResult<()> {
     let mut to_remove = Vec::new();
-    {
-        let mut installed = match metadata.v1_mut().entry(pkgid) {
-            Entry::Occupied(e) => e,
-            Entry::Vacant(..) => failure::bail!("package `{}` is not installed", pkgid),
-        };
+    let installed = match tracker.installed_bins(pkgid) {
+        Some(bins) => bins.clone(),
+        None => bail!("package `{}` is not installed", pkgid),
+    };
 
-        let dst = crate_metadata.parent().join("bin");
-        for bin in installed.get() {
-            let bin = dst.join(bin);
-            if fs::metadata(&bin).is_err() {
-                failure::bail!(
-                    "corrupt metadata, `{}` does not exist when it should",
-                    bin.display()
-                )
-            }
-        }
-
-        let bins = bins
-            .iter()
-            .map(|s| {
-                if s.ends_with(env::consts::EXE_SUFFIX) {
-                    s.to_string()
-                } else {
-                    format!("{}{}", s, env::consts::EXE_SUFFIX)
-                }
-            })
-            .collect::<Vec<_>>();
-
-        for bin in bins.iter() {
-            if !installed.get().contains(bin) {
-                failure::bail!("binary `{}` not installed as part of `{}`", bin, pkgid)
-            }
-        }
-
-        if bins.is_empty() {
-            to_remove.extend(installed.get().iter().map(|b| dst.join(b)));
-            installed.get_mut().clear();
-        } else {
-            for bin in bins.iter() {
-                to_remove.push(dst.join(bin));
-                installed.get_mut().remove(bin);
-            }
-        }
-        if installed.get().is_empty() {
-            installed.remove();
+    let dst = root.join("bin").into_path_unlocked();
+    for bin in &installed {
+        let bin = dst.join(bin);
+        if !bin.exists() {
+            bail!(
+                "corrupt metadata, `{}` does not exist when it should",
+                bin.display()
+            )
         }
     }
-    write_crate_list(crate_metadata, metadata)?;
+
+    let bins = bins
+        .iter()
+        .map(|s| {
+            if s.ends_with(env::consts::EXE_SUFFIX) {
+                s.to_string()
+            } else {
+                format!("{}{}", s, env::consts::EXE_SUFFIX)
+            }
+        })
+        .collect::<BTreeSet<_>>();
+
+    for bin in bins.iter() {
+        if !installed.contains(bin) {
+            bail!("binary `{}` not installed as part of `{}`", bin, pkgid)
+        }
+    }
+
+    if bins.is_empty() {
+        to_remove.extend(installed.iter().map(|b| dst.join(b)));
+        tracker.remove(pkgid, &installed);
+    } else {
+        for bin in bins.iter() {
+            to_remove.push(dst.join(bin));
+        }
+        tracker.remove(pkgid, &bins);
+    }
+    tracker.save()?;
     for bin in to_remove {
         config.shell().status("Removing", bin.display())?;
         paths::remove_file(bin)?;
