@@ -10,6 +10,7 @@ use failure::{Context, Error, Fail};
 use log::trace;
 
 use crate::core::{TargetKind, Workspace};
+use crate::ops::CompileOptions;
 
 pub type CargoResult<T> = failure::Fallible<T>; // Alex's body isn't quite ready to give up "Result"
 
@@ -188,14 +189,14 @@ impl CargoTestError {
         }
     }
 
-    pub fn hint(&self, ws: &Workspace<'_>) -> String {
+    pub fn hint(&self, ws: &Workspace<'_>, opts: &CompileOptions<'_>) -> String {
         match self.test {
             Test::UnitTest {
                 ref kind,
                 ref name,
                 ref pkg_name,
             } => {
-                let pkg_info = if ws.members().count() > 1 && ws.is_virtual() {
+                let pkg_info = if opts.spec.needs_spec_flag(ws) {
                     format!("-p {} ", pkg_name)
                 } else {
                     String::new()
@@ -387,3 +388,11 @@ pub fn internal<S: fmt::Display>(error: S) -> failure::Error {
 fn _internal(error: &dyn fmt::Display) -> failure::Error {
     Internal::new(failure::format_err!("{}", error)).into()
 }
+
+pub fn display_causes(error: &Error) -> String {
+    error
+        .iter_chain()
+        .map(|e| e.to_string())
+        .collect::<Vec<_>>()
+        .join("\nCaused by:\n  ")
+}
