@@ -3,12 +3,11 @@ use std::fs::{self, File};
 use std::io;
 use std::io::prelude::*;
 use std::thread;
-use std::time::Duration;
 
 use crate::support::paths::CargoPathExt;
 use crate::support::registry::Package;
 use crate::support::{basic_manifest, cross_compile, project};
-use crate::support::{rustc_host, sleep_ms};
+use crate::support::{rustc_host, sleep_ms, slow_cpu_multiplier};
 use cargo::util::paths::remove_dir_all;
 
 #[test]
@@ -36,7 +35,9 @@ fn custom_build_script_failed() {
 [RUNNING] `rustc --crate-name build_script_build build.rs --color never --crate-type bin [..]`
 [RUNNING] `[..]/build-script-build`
 [ERROR] failed to run custom build command for `foo v0.5.0 ([CWD])`
-process didn't exit successfully: `[..]/build-script-build` (exit code: 101)",
+
+Caused by:
+  process didn't exit successfully: `[..]/build-script-build` (exit code: 101)",
         )
         .run();
 }
@@ -232,7 +233,7 @@ fn custom_build_script_rustc_flags() {
         "#,
         ).build();
 
-    // TODO: TEST FAILS BECAUSE OF WRONG STDOUT (but otherwise, the build works)
+    // TODO: TEST FAILS BECAUSE OF WRONG STDOUT (but otherwise, the build works).
     p.cargo("build --verbose")
         .with_status(101)
         .with_stderr(
@@ -242,7 +243,7 @@ fn custom_build_script_rustc_flags() {
         -C metadata=[..] \
         -C extra-filename=-[..] \
         --out-dir [CWD]/target \
-        --emit=dep-info,link \
+        --emit=[..]link \
         -L [CWD]/target \
         -L [CWD]/target/deps`
 ",
@@ -1016,19 +1017,19 @@ fn build_cmd_with_a_build_cmd() {
 [RUNNING] `rustc [..] a/build.rs [..] --extern b=[..]`
 [RUNNING] `[..]/a-[..]/build-script-build`
 [RUNNING] `rustc --crate-name a [..]lib.rs --color never --crate-type lib \
-    --emit=dep-info,link -C debuginfo=2 \
+    --emit=[..]link -C debuginfo=2 \
     -C metadata=[..] \
     --out-dir [..]target/debug/deps \
     -L [..]target/debug/deps`
 [COMPILING] foo v0.5.0 ([CWD])
 [RUNNING] `rustc --crate-name build_script_build build.rs --color never --crate-type bin \
-    --emit=dep-info,link \
+    --emit=[..]link \
     -C debuginfo=2 -C metadata=[..] --out-dir [..] \
     -L [..]target/debug/deps \
     --extern a=[..]liba[..].rlib`
 [RUNNING] `[..]/foo-[..]/build-script-build`
 [RUNNING] `rustc --crate-name foo [..]lib.rs --color never --crate-type lib \
-    --emit=dep-info,link -C debuginfo=2 \
+    --emit=[..]link -C debuginfo=2 \
     -C metadata=[..] \
     --out-dir [..] \
     -L [..]target/debug/deps`
@@ -1548,13 +1549,13 @@ fn build_script_with_dynamic_native_dependency() {
 
     build
         .cargo("build -v")
-        .env("RUST_LOG", "cargo::ops::cargo_rustc")
+        .env("CARGO_LOG", "cargo::ops::cargo_rustc")
         .run();
 
     let root = build.root().join("target").join("debug");
     foo.cargo("build -v")
         .env("BUILDER_ROOT", root)
-        .env("RUST_LOG", "cargo::ops::cargo_rustc")
+        .env("CARGO_LOG", "cargo::ops::cargo_rustc")
         .run();
 }
 
@@ -2407,7 +2408,6 @@ fn fresh_builds_possible_with_link_libs() {
         .run();
 
     p.cargo("build -v")
-        .env("RUST_LOG", "cargo::ops::cargo_rustc::fingerprint=info")
         .with_stderr(
             "\
 [FRESH] foo v0.5.0 ([..])
@@ -2462,7 +2462,7 @@ fn fresh_builds_possible_with_multiple_metadata_overrides() {
         .run();
 
     p.cargo("build -v")
-        .env("RUST_LOG", "cargo::ops::cargo_rustc::fingerprint=info")
+        .env("CARGO_LOG", "cargo::ops::cargo_rustc::fingerprint=info")
         .with_stderr(
             "\
 [FRESH] foo v0.5.0 ([..])
@@ -3171,9 +3171,14 @@ fn switch_features_rerun() {
         )
         .build();
 
-    p.cargo("run -v --features=foo").with_stdout("foo\n").run();
-    p.cargo("run -v").with_stdout("bar\n").run();
-    p.cargo("run -v --features=foo").with_stdout("foo\n").run();
+    p.cargo("build -v --features=foo").run();
+    p.rename_run("foo", "with_foo").with_stdout("foo\n").run();
+    p.cargo("build -v").run();
+    p.rename_run("foo", "without_foo")
+        .with_stdout("bar\n")
+        .run();
+    p.cargo("build -v --features=foo").run();
+    p.rename_run("foo", "with_foo2").with_stdout("foo\n").run();
 }
 
 #[test]
@@ -3514,7 +3519,7 @@ fn _rename_with_link_search_path(cross: bool) {
     let p2 = p2.build();
 
     // Move the output `libfoo.so` into the directory of `p2`, and then delete
-    // the `p` project. On OSX the `libfoo.dylib` artifact references the
+    // the `p` project. On macOS, the `libfoo.dylib` artifact references the
     // original path in `p` so we want to make sure that it can't find it (hence
     // the deletion).
     let root = if cross {
@@ -3566,7 +3571,7 @@ fn _rename_with_link_search_path(cross: bool) {
             panic!("failed to rename: {}", error);
         }
         println!("assuming {} is spurious, waiting to try again", error);
-        thread::sleep(Duration::from_millis(100));
+        thread::sleep(slow_cpu_multiplier(100));
     }
 
     p2.cargo(&format!("run{}", target_arg))
@@ -3692,3 +3697,142 @@ fn optional_build_dep_and_required_normal_dep() {
         )
         .run();
 }
+
+#[test]
+fn using_rerun_if_changed_does_not_rebuild() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "0.1.0"
+                authors = []
+            "#,
+        )
+        .file(
+            "build.rs",
+            r#"
+                fn main() {
+                    println!("cargo:rerun-if-changed=build.rs");
+                }
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("build").run();
+    p.cargo("build").with_stderr("[FINISHED] [..]").run();
+}
+
+#[test]
+fn links_interrupted_can_restart() {
+    // Test for a `links` dependent build script getting canceled and then
+    // restarted. Steps:
+    // 1. Build to establish fingerprints.
+    // 2. Change something (an env var in this case) that triggers the
+    //    dependent build script to run again. Kill the top-level build script
+    //    while it is running (such as hitting Ctrl-C).
+    // 3. Run the build again, it should re-run the build script.
+    let bar = project()
+        .at("bar")
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "bar"
+            version = "0.5.0"
+            authors = []
+            links = "foo"
+            build = "build.rs"
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .file(
+            "build.rs",
+            r#"
+            fn main() {
+                println!("cargo:rerun-if-env-changed=SOMEVAR");
+            }
+            "#,
+        )
+        .build();
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            &format!(
+                r#"
+                [package]
+                name = "foo"
+                version = "0.5.0"
+                authors = []
+                build = "build.rs"
+
+                [dependencies.bar]
+                path = '{}'
+                "#,
+                bar.root().display()
+            ),
+        )
+        .file("src/lib.rs", "")
+        .file(
+            "build.rs",
+            r#"
+            use std::env;
+            fn main() {
+                println!("cargo:rebuild-if-changed=build.rs");
+                if std::path::Path::new("abort").exists() {
+                    panic!("Crash!");
+                }
+            }
+            "#,
+        )
+        .build();
+
+    p.cargo("build").run();
+    // Simulate the user hitting Ctrl-C during a build.
+    p.change_file("abort", "");
+    // Set SOMEVAR to trigger a rebuild.
+    p.cargo("build")
+        .env("SOMEVAR", "1")
+        .with_stderr_contains("[..]Crash![..]")
+        .with_status(101)
+        .run();
+    fs::remove_file(p.root().join("abort")).unwrap();
+    // Try again without aborting the script.
+    // ***This is currently broken, the script does not re-run.
+    p.cargo("build -v")
+        .env("SOMEVAR", "1")
+        .with_stderr_contains("[RUNNING] [..]/foo-[..]/build-script-build[..]")
+        .run();
+}
+
+#[test]
+#[cfg(unix)]
+fn build_script_scan_eacces() {
+    // build.rs causes a scan of the whole project, which can be a problem if
+    // a directory is not accessible.
+    use std::os::unix::fs::PermissionsExt;
+    let p = project()
+        .file("src/lib.rs", "")
+        .file("build.rs", "fn main() {}")
+        .file("secrets/stuff", "")
+        .build();
+    let path = p.root().join("secrets");
+    fs::set_permissions(&path, fs::Permissions::from_mode(0)).unwrap();
+    // "Caused by" is a string from libc such as the following:
+    //   Permission denied (os error 13)
+    p.cargo("build")
+        .with_stderr(
+            "\
+[ERROR] cannot read \"[..]/foo/secrets\"
+
+Caused by:
+  [..]
+",
+        )
+        .with_status(101)
+        .run();
+    fs::set_permissions(&path, fs::Permissions::from_mode(0o755)).unwrap();
+}
