@@ -1,11 +1,7 @@
-use crate::support::{basic_lib_manifest, is_nightly, project};
+use crate::support::{basic_lib_manifest, project};
 
 #[test]
 fn edition_works_for_build_script() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
@@ -32,8 +28,5 @@ fn edition_works_for_build_script() {
         .file("a/src/lib.rs", "pub fn foo() {}")
         .build();
 
-    p.cargo("build -v")
-        .masquerade_as_nightly_cargo()
-        .with_status(0)
-        .run();
+    p.cargo("build -v").run();
 }
