@@ -13,7 +13,7 @@ fn rustc_info_cache() {
     let update = "[..]updated rustc info cache[..]";
 
     p.cargo("build")
-        .env("RUST_LOG", "cargo::util::rustc=info")
+        .env("CARGO_LOG", "cargo::util::rustc=info")
         .with_stderr_contains("[..]failed to read rustc info cache[..]")
         .with_stderr_contains(miss)
         .with_stderr_does_not_contain(hit)
@@ -21,7 +21,7 @@ fn rustc_info_cache() {
         .run();
 
     p.cargo("build")
-        .env("RUST_LOG", "cargo::util::rustc=info")
+        .env("CARGO_LOG", "cargo::util::rustc=info")
         .with_stderr_contains("[..]reusing existing rustc info cache[..]")
         .with_stderr_contains(hit)
         .with_stderr_does_not_contain(miss)
@@ -29,7 +29,7 @@ fn rustc_info_cache() {
         .run();
 
     p.cargo("build")
-        .env("RUST_LOG", "cargo::util::rustc=info")
+        .env("CARGO_LOG", "cargo::util::rustc=info")
         .env("CARGO_CACHE_RUSTC_INFO", "0")
         .with_stderr_contains("[..]rustc info cache disabled[..]")
         .with_stderr_does_not_contain(update)
@@ -63,7 +63,7 @@ fn rustc_info_cache() {
     };
 
     p.cargo("build")
-        .env("RUST_LOG", "cargo::util::rustc=info")
+        .env("CARGO_LOG", "cargo::util::rustc=info")
         .env("RUSTC", other_rustc.display().to_string())
         .with_stderr_contains("[..]different compiler, creating new rustc info cache[..]")
         .with_stderr_contains(miss)
@@ -72,7 +72,7 @@ fn rustc_info_cache() {
         .run();
 
     p.cargo("build")
-        .env("RUST_LOG", "cargo::util::rustc=info")
+        .env("CARGO_LOG", "cargo::util::rustc=info")
         .env("RUSTC", other_rustc.display().to_string())
         .with_stderr_contains("[..]reusing existing rustc info cache[..]")
         .with_stderr_contains(hit)
@@ -83,7 +83,7 @@ fn rustc_info_cache() {
     other_rustc.move_into_the_future();
 
     p.cargo("build")
-        .env("RUST_LOG", "cargo::util::rustc=info")
+        .env("CARGO_LOG", "cargo::util::rustc=info")
         .env("RUSTC", other_rustc.display().to_string())
         .with_stderr_contains("[..]different compiler, creating new rustc info cache[..]")
         .with_stderr_contains(miss)
@@ -92,7 +92,7 @@ fn rustc_info_cache() {
         .run();
 
     p.cargo("build")
-        .env("RUST_LOG", "cargo::util::rustc=info")
+        .env("CARGO_LOG", "cargo::util::rustc=info")
         .env("RUSTC", other_rustc.display().to_string())
         .with_stderr_contains("[..]reusing existing rustc info cache[..]")
         .with_stderr_contains(hit)
