@@ -2,7 +2,7 @@
 //! in the output, their arguments are quoted properly
 //! so that the command can be run in a terminal
 
-use support::project;
+use crate::support::project;
 
 #[test]
 fn features_are_quoted() {
@@ -19,7 +19,8 @@ fn features_are_quoted() {
             some_feature = []
             default = ["some_feature"]
             "#,
-        ).file("src/main.rs", "fn main() {error}")
+        )
+        .file("src/main.rs", "fn main() {error}")
         .build();
 
     p.cargo("check -v")
