@@ -1,13 +1,13 @@
 /*
-# Introduction To `support`
+# Introduction to `support`.
 
 Cargo has a wide variety of integration tests that execute the `cargo` binary
-and verify its behavior.  The `support` module contains many helpers to make
+and verify its behavior. The `support` module contains many helpers to make
 this process easy.
 
 The general form of a test involves creating a "project", running cargo, and
-checking the result.  Projects are created with the `ProjectBuilder` where you
-specify some files to create.  The general form looks like this:
+checking the result. Projects are created with the `ProjectBuilder` where you
+specify some files to create. The general form looks like this:
 
 ```
 let p = project()
@@ -35,8 +35,8 @@ p.cargo("run --bin foo")
 
 The project creates a mini sandbox under the "cargo integration test"
 directory with each test getting a separate directory such as
-`/path/to/cargo/target/cit/t123/`.  Each project appears as a separate
-directory.  There is also an empty `home` directory created that will be used
+`/path/to/cargo/target/cit/t123/`. Each project appears as a separate
+directory. There is also an empty `home` directory created that will be used
 as a home directory instead of your normal home directory.
 
 See `support::lines_match` for an explanation of the string pattern matching.
@@ -60,6 +60,7 @@ rust, like this:
 
 ```
 if !is_nightly() {
+    // Add a comment here explaining why this is necessary.
     return;
 }
 ```
@@ -69,8 +70,8 @@ if !is_nightly() {
 When checking output, use `/` for paths even on Windows: the actual output
 of `\` on Windows will be replaced with `/`.
 
-Be careful when executing binaries on Windows.  You should not rename, delete,
-or overwrite a binary immediately after running it.  Under some conditions
+Be careful when executing binaries on Windows. You should not rename, delete,
+or overwrite a binary immediately after running it. Under some conditions
 Windows will fail with errors like "directory not empty" or "failed to remove"
 or "access is denied".
 
@@ -242,7 +243,7 @@ impl ProjectBuilder {
         self
     }
 
-    /// Add a file to the project.
+    /// Adds a file to the project.
     pub fn file<B: AsRef<Path>>(mut self, path: B, body: &str) -> Self {
         self._file(path.as_ref(), body);
         self
@@ -253,7 +254,7 @@ impl ProjectBuilder {
             .push(FileBuilder::new(self.root.root().join(path), body));
     }
 
-    /// Add a symlink to the project.
+    /// Adds a symlink to the project.
     pub fn symlink<T: AsRef<Path>>(mut self, dst: T, src: T) -> Self {
         self.symlinks.push(SymlinkBuilder::new(
             self.root.root().join(dst),
@@ -267,7 +268,7 @@ impl ProjectBuilder {
         self
     }
 
-    /// Create the project.
+    /// Creates the project.
     pub fn build(mut self) -> Project {
         // First, clean the directory if it already exists
         self.rm_root();
@@ -370,12 +371,20 @@ impl Project {
         ))
     }
 
-    /// Change the contents of an existing file.
+    /// Returns an iterator of paths matching the glob pattern, which is
+    /// relative to the project root.
+    pub fn glob<P: AsRef<Path>>(&self, pattern: P) -> glob::Paths {
+        let pattern = self.root().join(pattern);
+        glob::glob(pattern.to_str().expect("failed to convert pattern to str"))
+            .expect("failed to glob")
+    }
+
+    /// Changes the contents of an existing file.
     pub fn change_file(&self, path: &str, body: &str) {
         FileBuilder::new(self.root().join(path), body).mk()
     }
 
-    /// Create a `ProcessBuilder` to run a program in the project
+    /// Creates a `ProcessBuilder` to run a program in the project
     /// and wrap it in an Execs to assert on the execution.
     /// Example:
     ///         p.process(&p.bin("foo"))
@@ -387,7 +396,7 @@ impl Project {
         execs().with_process_builder(p)
     }
 
-    /// Create a `ProcessBuilder` to run cargo.
+    /// Creates a `ProcessBuilder` to run cargo.
     /// Arguments can be separated by spaces.
     /// Example:
     ///     p.cargo("build --bin foo").run();
@@ -399,6 +408,27 @@ impl Project {
         execs
     }
 
+    /// Safely run a process after `cargo build`.
+    ///
+    /// Windows has a problem where a process cannot be reliably
+    /// be replaced, removed, or renamed immediately after executing it.
+    /// The action may fail (with errors like Access is denied), or
+    /// it may succeed, but future attempts to use the same filename
+    /// will fail with "Already Exists".
+    ///
+    /// If you have a test that needs to do `cargo run` multiple
+    /// times, you should instead use `cargo build` and use this
+    /// method to run the executable. Each time you call this,
+    /// use a new name for `dst`.
+    /// See rust-lang/cargo#5481.
+    pub fn rename_run(&self, src: &str, dst: &str) -> Execs {
+        let src = self.bin(src);
+        let dst = self.bin(dst);
+        fs::rename(&src, &dst)
+            .unwrap_or_else(|e| panic!("Failed to rename `{:?}` to `{:?}`: {}", src, dst, e));
+        self.process(dst)
+    }
+
     /// Returns the contents of `Cargo.lock`.
     pub fn read_lockfile(&self) -> String {
         self.read_file("Cargo.lock")
@@ -485,10 +515,10 @@ pub fn main_file(println: &str, deps: &[&str]) -> String {
     }
 
     buf.push_str("fn main() { println!(");
-    buf.push_str(&println);
+    buf.push_str(println);
     buf.push_str("); }\n");
 
-    buf.to_string()
+    buf
 }
 
 trait ErrMsg<T> {
@@ -549,6 +579,7 @@ pub struct Execs {
     expect_stderr_not_contains: Vec<String>,
     expect_stderr_unordered: Vec<String>,
     expect_neither_contains: Vec<String>,
+    expect_stderr_with_without: Vec<(Vec<String>, Vec<String>)>,
     expect_json: Option<Vec<Value>>,
     expect_json_contains_unordered: Vec<Value>,
     stream_output: bool,
@@ -560,21 +591,21 @@ impl Execs {
         self
     }
 
-    /// Verify that stdout is equal to the given lines.
+    /// Verifies that stdout is equal to the given lines.
     /// See `lines_match` for supported patterns.
     pub fn with_stdout<S: ToString>(&mut self, expected: S) -> &mut Self {
         self.expect_stdout = Some(expected.to_string());
         self
     }
 
-    /// Verify that stderr is equal to the given lines.
+    /// Verifies that stderr is equal to the given lines.
     /// See `lines_match` for supported patterns.
     pub fn with_stderr<S: ToString>(&mut self, expected: S) -> &mut Self {
         self.expect_stderr = Some(expected.to_string());
         self
     }
 
-    /// Verify the exit code from the process.
+    /// Verifies the exit code from the process.
     ///
     /// This is not necessary if the expected exit code is `0`.
     pub fn with_status(&mut self, expected: i32) -> &mut Self {
@@ -582,7 +613,7 @@ impl Execs {
         self
     }
 
-    /// Remove exit code check for the process.
+    /// Removes exit code check for the process.
     ///
     /// By default, the expected exit code is `0`.
     pub fn without_status(&mut self) -> &mut Self {
@@ -590,7 +621,7 @@ impl Execs {
         self
     }
 
-    /// Verify that stdout contains the given contiguous lines somewhere in
+    /// Verifies that stdout contains the given contiguous lines somewhere in
     /// its output.
     /// See `lines_match` for supported patterns.
     pub fn with_stdout_contains<S: ToString>(&mut self, expected: S) -> &mut Self {
@@ -598,7 +629,7 @@ impl Execs {
         self
     }
 
-    /// Verify that stderr contains the given contiguous lines somewhere in
+    /// Verifies that stderr contains the given contiguous lines somewhere in
     /// its output.
     /// See `lines_match` for supported patterns.
     pub fn with_stderr_contains<S: ToString>(&mut self, expected: S) -> &mut Self {
@@ -606,7 +637,7 @@ impl Execs {
         self
     }
 
-    /// Verify that either stdout or stderr contains the given contiguous
+    /// Verifies that either stdout or stderr contains the given contiguous
     /// lines somewhere in its output.
     /// See `lines_match` for supported patterns.
     pub fn with_either_contains<S: ToString>(&mut self, expected: S) -> &mut Self {
@@ -614,7 +645,7 @@ impl Execs {
         self
     }
 
-    /// Verify that stdout contains the given contiguous lines somewhere in
+    /// Verifies that stdout contains the given contiguous lines somewhere in
     /// its output, and should be repeated `number` times.
     /// See `lines_match` for supported patterns.
     pub fn with_stdout_contains_n<S: ToString>(&mut self, expected: S, number: usize) -> &mut Self {
@@ -623,7 +654,7 @@ impl Execs {
         self
     }
 
-    /// Verify that stdout does not contain the given contiguous lines.
+    /// Verifies that stdout does not contain the given contiguous lines.
     /// See `lines_match` for supported patterns.
     /// See note on `with_stderr_does_not_contain`.
     pub fn with_stdout_does_not_contain<S: ToString>(&mut self, expected: S) -> &mut Self {
@@ -631,11 +662,11 @@ impl Execs {
         self
     }
 
-    /// Verify that stderr does not contain the given contiguous lines.
+    /// Verifies that stderr does not contain the given contiguous lines.
     /// See `lines_match` for supported patterns.
     ///
     /// Care should be taken when using this method because there is a
-    /// limitless number of possible things that *won't* appear.  A typo means
+    /// limitless number of possible things that *won't* appear. A typo means
     /// your test will pass without verifying the correct behavior. If
     /// possible, write the test first so that it fails, and then implement
     /// your fix/feature to make it pass.
@@ -644,7 +675,7 @@ impl Execs {
         self
     }
 
-    /// Verify that all of the stderr output is equal to the given lines,
+    /// Verifies that all of the stderr output is equal to the given lines,
     /// ignoring the order of the lines.
     /// See `lines_match` for supported patterns.
     /// This is useful when checking the output of `cargo build -v` since
@@ -654,9 +685,11 @@ impl Execs {
     ///
     /// Be careful when using patterns such as `[..]`, because you may end up
     /// with multiple lines that might match, and this is not smart enough to
-    /// do anything like longest-match.  For example, avoid something like:
+    /// do anything like longest-match. For example, avoid something like:
+    ///
     ///     [RUNNING] `rustc [..]
     ///     [RUNNING] `rustc --crate-name foo [..]
+    ///
     /// This will randomly fail if the other crate name is `bar`, and the
     /// order changes.
     pub fn with_stderr_unordered<S: ToString>(&mut self, expected: S) -> &mut Self {
@@ -664,7 +697,38 @@ impl Execs {
         self
     }
 
-    /// Verify the JSON output matches the given JSON.
+    /// Verify that a particular line appears in stderr with and without the
+    /// given substrings. Exactly one line must match.
+    ///
+    /// The substrings are matched as `contains`. Example:
+    ///
+    /// ```no_run
+    /// execs.with_stderr_line_without(
+    ///     &[
+    ///         "[RUNNING] `rustc --crate-name build_script_build",
+    ///         "-C opt-level=3",
+    ///     ],
+    ///     &["-C debuginfo", "-C incremental"],
+    /// )
+    /// ```
+    ///
+    /// This will check that a build line includes `-C opt-level=3` but does
+    /// not contain `-C debuginfo` or `-C incremental`.
+    ///
+    /// Be careful writing the `without` fragments, see note in
+    /// `with_stderr_does_not_contain`.
+    pub fn with_stderr_line_without<S: ToString>(
+        &mut self,
+        with: &[S],
+        without: &[S],
+    ) -> &mut Self {
+        let with = with.iter().map(|s| s.to_string()).collect();
+        let without = without.iter().map(|s| s.to_string()).collect();
+        self.expect_stderr_with_without.push((with, without));
+        self
+    }
+
+    /// Verifies the JSON output matches the given JSON.
     /// Typically used when testing cargo commands that emit JSON.
     /// Each separate JSON object should be separated by a blank line.
     /// Example:
@@ -690,7 +754,7 @@ impl Execs {
         self
     }
 
-    /// Verify JSON output contains the given objects (in any order) somewhere
+    /// Verifies JSON output contains the given objects (in any order) somewhere
     /// in its output.
     ///
     /// CAUTION: Be very careful when using this. Make sure every object is
@@ -726,7 +790,12 @@ impl Execs {
 
     pub fn cwd<T: AsRef<OsStr>>(&mut self, path: T) -> &mut Self {
         if let Some(ref mut p) = self.process_builder {
-            p.cwd(path);
+            if let Some(cwd) = p.get_cwd() {
+                let new_path = cwd.join(path.as_ref());
+                p.cwd(new_path);
+            } else {
+                p.cwd(path);
+            }
         }
         self
     }
@@ -794,6 +863,7 @@ impl Execs {
             && self.expect_stderr_not_contains.is_empty()
             && self.expect_stderr_unordered.is_empty()
             && self.expect_neither_contains.is_empty()
+            && self.expect_stderr_with_without.is_empty()
             && self.expect_json.is_none()
             && self.expect_json_contains_unordered.is_empty()
         {
@@ -890,7 +960,7 @@ impl Execs {
         }
         for &(ref expect, number) in self.expect_stdout_contains_n.iter() {
             self.match_std(
-                Some(&expect),
+                Some(expect),
                 &actual.stdout,
                 "stdout",
                 &actual.stderr,
@@ -968,6 +1038,10 @@ impl Execs {
             }
         }
 
+        for (with, without) in self.expect_stderr_with_without.iter() {
+            self.match_with_without(&actual.stderr, with, without)?;
+        }
+
         if let Some(ref objects) = self.expect_json {
             let stdout = str::from_utf8(&actual.stdout)
                 .map_err(|_| "stdout was not utf8 encoded".to_owned())?;
@@ -1027,6 +1101,32 @@ impl Execs {
         )
     }
 
+    fn normalize_actual(&self, description: &str, actual: &[u8]) -> Result<String, String> {
+        let actual = match str::from_utf8(actual) {
+            Err(..) => return Err(format!("{} was not utf8 encoded", description)),
+            Ok(actual) => actual,
+        };
+        // Let's not deal with \r\n vs \n on windows...
+        let actual = actual.replace("\r", "");
+        let actual = actual.replace("\t", "<tab>");
+        Ok(actual)
+    }
+
+    fn replace_expected(&self, expected: &str) -> String {
+        // Do the template replacements on the expected string.
+        let replaced = match self.process_builder {
+            None => expected.to_string(),
+            Some(ref p) => match p.get_cwd() {
+                None => expected.to_string(),
+                Some(cwd) => expected.replace("[CWD]", &cwd.display().to_string()),
+            },
+        };
+
+        // On Windows, we need to use a wildcard for the drive,
+        // because we don't actually know what it will be.
+        replaced.replace("[ROOT]", if cfg!(windows) { r#"[..]:\"# } else { "/" })
+    }
+
     fn match_std(
         &self,
         expected: Option<&String>,
@@ -1036,33 +1136,11 @@ impl Execs {
         kind: MatchKind,
     ) -> MatchResult {
         let out = match expected {
-            Some(out) => {
-                // Do the template replacements on the expected string.
-                let replaced = match self.process_builder {
-                    None => out.to_string(),
-                    Some(ref p) => match p.get_cwd() {
-                        None => out.to_string(),
-                        Some(cwd) => out.replace("[CWD]", &cwd.display().to_string()),
-                    },
-                };
-
-                // On Windows, we need to use a wildcard for the drive,
-                // because we don't actually know what it will be.
-                let replaced =
-                    replaced.replace("[ROOT]", if cfg!(windows) { r#"[..]:\"# } else { "/" });
-
-                replaced
-            }
+            Some(out) => self.replace_expected(out),
             None => return Ok(()),
         };
 
-        let actual = match str::from_utf8(actual) {
-            Err(..) => return Err(format!("{} was not utf8 encoded", description)),
-            Ok(actual) => actual,
-        };
-        // Let's not deal with \r\n vs \n on windows...
-        let actual = actual.replace("\r", "");
-        let actual = actual.replace("\t", "<tab>");
+        let actual = self.normalize_actual(description, actual)?;
 
         match kind {
             MatchKind::Exact => {
@@ -1186,6 +1264,47 @@ impl Execs {
         }
     }
 
+    fn match_with_without(
+        &self,
+        actual: &[u8],
+        with: &[String],
+        without: &[String],
+    ) -> MatchResult {
+        let actual = self.normalize_actual("stderr", actual)?;
+        let contains = |s, line| {
+            let mut s = self.replace_expected(s);
+            s.insert_str(0, "[..]");
+            s.push_str("[..]");
+            lines_match(&s, line)
+        };
+        let matches: Vec<&str> = actual
+            .lines()
+            .filter(|line| with.iter().all(|with| contains(with, line)))
+            .filter(|line| !without.iter().any(|without| contains(without, line)))
+            .collect();
+        match matches.len() {
+            0 => Err(format!(
+                "Could not find expected line in output.\n\
+                 With contents: {:?}\n\
+                 Without contents: {:?}\n\
+                 Actual stderr:\n\
+                 {}\n",
+                with, without, actual
+            )),
+            1 => Ok(()),
+            _ => Err(format!(
+                "Found multiple matching lines, but only expected one.\n\
+                 With contents: {:?}\n\
+                 Without contents: {:?}\n\
+                 Matching lines:\n\
+                 {}\n",
+                with,
+                without,
+                matches.join("\n")
+            )),
+        }
+    }
+
     fn match_json(&self, expected: &Value, line: &str) -> MatchResult {
         let actual = match line.parse() {
             Err(e) => return Err(format!("invalid json, {}:\n`{}`", e, line)),
@@ -1210,7 +1329,7 @@ impl Execs {
             .enumerate()
             .filter_map(|(i, (a, e))| match (a, e) {
                 (Some(a), Some(e)) => {
-                    if lines_match(&e, &a) {
+                    if lines_match(e, a) {
                         None
                     } else {
                         Some(format!("{:3} - |{}|\n    + |{}|\n", i, e, a))
@@ -1241,7 +1360,7 @@ enum MatchKind {
     Unordered,
 }
 
-/// Compare a line with an expected pattern.
+/// Compares a line with an expected pattern.
 /// - Use `[..]` as a wildcard to match 0 or more characters on the same line
 ///   (similar to `.*` in a regex).
 /// - Use `[EXE]` to optionally add `.exe` on Windows (empty string on other
@@ -1249,6 +1368,8 @@ enum MatchKind {
 /// - There is a wide range of macros (such as `[COMPILING]` or `[WARNING]`)
 ///   to match cargo's "status" output and allows you to ignore the alignment.
 ///   See `substitute_macros` for a complete list of macros.
+/// - `[ROOT]` is `/` or `[..]:\` on Windows.
+/// - `[CWD]` is the working directory of the process that was run.
 pub fn lines_match(expected: &str, actual: &str) -> bool {
     // Let's not deal with / vs \ (windows...)
     // First replace backslash-escaped backslashes with forward slashes
@@ -1284,12 +1405,12 @@ fn lines_match_works() {
 }
 
 /// Compares JSON object for approximate equality.
-/// You can use `[..]` wildcard in strings (useful for OS dependent things such
-/// as paths).  You can use a `"{...}"` string literal as a wildcard for
+/// You can use `[..]` wildcard in strings (useful for OS-dependent things such
+/// as paths). You can use a `"{...}"` string literal as a wildcard for
 /// arbitrary nested JSON (useful for parts of object emitted by other programs
-/// (e.g. rustc) rather than Cargo itself).  Arrays are sorted before comparison.
+/// (e.g., rustc) rather than Cargo itself). Arrays are sorted before comparison.
 pub fn find_json_mismatch(expected: &Value, actual: &Value) -> Result<(), String> {
-    match find_json_mismatch_r(expected, &actual) {
+    match find_json_mismatch_r(expected, actual) {
         Some((expected_part, actual_part)) => Err(format!(
             "JSON mismatch\nExpected:\n{}\nWas:\n{}\nExpected part:\n{}\nActual part:\n{}\n",
             serde_json::to_string_pretty(expected).unwrap(),
@@ -1330,7 +1451,7 @@ fn find_json_mismatch_r<'a>(
 
             if !l.is_empty() {
                 assert!(!r.is_empty());
-                Some((&l[0], &r[0]))
+                Some((l[0], r[0]))
             } else {
                 assert_eq!(r.len(), 0);
                 None
@@ -1348,7 +1469,7 @@ fn find_json_mismatch_r<'a>(
                 .nth(0)
         }
         (&Null, &Null) => None,
-        // magic string literal "{...}" acts as wildcard for any sub-JSON
+        // Magic string literal `"{...}"` acts as wildcard for any sub-JSON.
         (&String(ref l), _) if l == "{...}" => None,
         _ => Some((expected, actual)),
     }
@@ -1401,6 +1522,7 @@ pub fn execs() -> Execs {
         expect_stderr_not_contains: Vec::new(),
         expect_stderr_unordered: Vec::new(),
         expect_neither_contains: Vec::new(),
+        expect_stderr_with_without: Vec::new(),
         expect_json: None,
         expect_json_contains_unordered: Vec::new(),
         stream_output: false,
@@ -1494,7 +1616,11 @@ fn substitute_macros(input: &str) -> String {
         ("[UNPACKING]", "   Unpacking"),
         ("[SUMMARY]", "     Summary"),
         ("[FIXING]", "      Fixing"),
-        ("[EXE]", if cfg!(windows) { ".exe" } else { "" }),
+        ("[EXE]", env::consts::EXE_SUFFIX),
+        ("[IGNORED]", "     Ignored"),
+        ("[INSTALLED]", "   Installed"),
+        ("[REPLACED]", "    Replaced"),
+        ("[NOTE]", "        Note"),
     ];
     let mut result = input.to_owned();
     for &(pat, subst) in &macros {
@@ -1534,7 +1660,7 @@ fn _process(t: &OsStr) -> cargo::util::ProcessBuilder {
         .env("HOME", paths::home())
         .env("CARGO_HOME", paths::home().join(".cargo"))
         .env("__CARGO_TEST_ROOT", paths::root())
-        // Force cargo to think it's on the stable channel for all tests, this
+        // Force Cargo to think it's on the stable channel for all tests, this
         // should hopefully not surprise us as we add cargo features over time and
         // cargo rides the trains.
         .env("__CARGO_TEST_CHANNEL_OVERRIDE_DO_NOT_USE_THIS", "stable")
@@ -1554,6 +1680,7 @@ fn _process(t: &OsStr) -> cargo::util::ProcessBuilder {
         .env_remove("XDG_CONFIG_HOME") // see #2345
         .env("GIT_CONFIG_NOSYSTEM", "1") // keep trying to sandbox ourselves
         .env_remove("EMAIL")
+        .env_remove("USER") // not set on some rust-lang docker images
         .env_remove("MFLAGS")
         .env_remove("MAKEFLAGS")
         .env_remove("CARGO_MAKEFLAGS")
@@ -1601,10 +1728,24 @@ pub fn sleep_ms(ms: u64) {
     ::std::thread::sleep(Duration::from_millis(ms));
 }
 
-/// Returns true if the local filesystem has low-resolution mtimes.
+/// Returns `true` if the local filesystem has low-resolution mtimes.
 pub fn is_coarse_mtime() -> bool {
-    // This should actually be a test that $CARGO_TARGET_DIR is on an HFS
+    // If the filetime crate is being used to emulate HFS then
+    // return `true`, without looking at the actual hardware.
+    cfg!(emulate_second_only_system) ||
+    // This should actually be a test that `$CARGO_TARGET_DIR` is on an HFS
     // filesystem, (or any filesystem with low-resolution mtimes). However,
     // that's tricky to detect, so for now just deal with CI.
     cfg!(target_os = "macos") && env::var("CI").is_ok()
 }
+
+/// Some CI setups are much slower then the equipment used by Cargo itself.
+/// Architectures that do not have a modern processor, hardware emulation, etc.
+/// This provides a way for those setups to increase the cut off for all the time based test.
+pub fn slow_cpu_multiplier(main: u64) -> Duration {
+    lazy_static::lazy_static! {
+        static ref SLOW_CPU_MULTIPLIER: u64 =
+            env::var("CARGO_TEST_SLOW_CPU_MULTIPLIER").ok().and_then(|m| m.parse().ok()).unwrap_or(1);
+    }
+    Duration::from_secs(*SLOW_CPU_MULTIPLIER * main)
+}
