@@ -2,7 +2,9 @@ use std::cmp::PartialEq;
 use std::cmp::{max, min};
 use std::collections::{BTreeMap, HashSet};
 use std::fmt;
-use std::time::{Duration, Instant};
+use std::time::Instant;
+
+use crate::support::slow_cpu_multiplier;
 
 use cargo::core::dependency::Kind;
 use cargo::core::resolver::{self, Method};
@@ -19,7 +21,7 @@ use proptest::string::string_regex;
 use proptest::test_runner::TestRunner;
 
 pub fn resolve(
-    pkg: &PackageId,
+    pkg: PackageId,
     deps: Vec<Dependency>,
     registry: &[Summary],
 ) -> CargoResult<Vec<PackageId>> {
@@ -27,17 +29,17 @@ pub fn resolve(
 }
 
 pub fn resolve_and_validated(
-    pkg: &PackageId,
+    pkg: PackageId,
     deps: Vec<Dependency>,
     registry: &[Summary],
 ) -> CargoResult<Vec<PackageId>> {
     let resolve = resolve_with_config_raw(pkg, deps, registry, None)?;
-    let mut stack = vec![pkg.clone()];
+    let mut stack = vec![pkg];
     let mut used = HashSet::new();
     let mut links = HashSet::new();
     while let Some(p) = stack.pop() {
         assert!(resolve.contains(&p));
-        if used.insert(p.clone()) {
+        if used.insert(p) {
             // in the tests all `links` crates end in `-sys`
             if p.name().ends_with("-sys") {
                 assert!(links.insert(p.name()));
@@ -46,7 +48,7 @@ pub fn resolve_and_validated(
                 for d in deps {
                     assert!(d.matches_id(dp));
                 }
-                dp.clone()
+                dp
             }));
         }
     }
@@ -56,7 +58,7 @@ pub fn resolve_and_validated(
 }
 
 pub fn resolve_with_config(
-    pkg: &PackageId,
+    pkg: PackageId,
     deps: Vec<Dependency>,
     registry: &[Summary],
     config: Option<&Config>,
@@ -66,12 +68,15 @@ pub fn resolve_with_config(
 }
 
 pub fn resolve_with_config_raw(
-    pkg: &PackageId,
+    pkg: PackageId,
     deps: Vec<Dependency>,
     registry: &[Summary],
     config: Option<&Config>,
 ) -> CargoResult<Resolve> {
-    struct MyRegistry<'a>(&'a [Summary]);
+    struct MyRegistry<'a> {
+        list: &'a [Summary],
+        used: HashSet<PackageId>,
+    };
     impl<'a> Registry for MyRegistry<'a> {
         fn query(
             &mut self,
@@ -79,8 +84,9 @@ pub fn resolve_with_config_raw(
             f: &mut dyn FnMut(Summary),
             fuzzy: bool,
         ) -> CargoResult<()> {
-            for summary in self.0.iter() {
+            for summary in self.list.iter() {
                 if fuzzy || dep.matches(summary) {
+                    self.used.insert(summary.package_id());
                     f(summary.clone());
                 }
             }
@@ -95,9 +101,30 @@ pub fn resolve_with_config_raw(
             false
         }
     }
-    let mut registry = MyRegistry(registry);
+    impl<'a> Drop for MyRegistry<'a> {
+        fn drop(&mut self) {
+            if std::thread::panicking() && self.list.len() != self.used.len() {
+                // we found a case that causes a panic and did not use all of the input.
+                // lets print the part of the input that was used for minimization.
+                println!(
+                    "{:?}",
+                    PrettyPrintRegistry(
+                        self.list
+                            .iter()
+                            .filter(|s| { self.used.contains(&s.package_id()) })
+                            .cloned()
+                            .collect()
+                    )
+                );
+            }
+        }
+    }
+    let mut registry = MyRegistry {
+        list: registry,
+        used: HashSet::new(),
+    };
     let summary = Summary::new(
-        pkg.clone(),
+        pkg,
         deps,
         &BTreeMap::<String, Vec<String>>::new(),
         None::<String>,
@@ -112,12 +139,12 @@ pub fn resolve_with_config_raw(
         &mut registry,
         &HashSet::new(),
         config,
-        false,
+        true,
     );
 
     // The largest test in our suite takes less then 30 sec.
     // So lets fail the test if we have ben running for two long.
-    assert!(start.elapsed() < Duration::from_secs(60));
+    assert!(start.elapsed() < slow_cpu_multiplier(60));
     resolve
 }
 
@@ -143,7 +170,7 @@ pub trait ToPkgId {
 
 impl ToPkgId for PackageId {
     fn to_pkgid(&self) -> PackageId {
-        self.clone()
+        *self
     }
 }
 
@@ -174,7 +201,7 @@ macro_rules! pkg {
 fn registry_loc() -> SourceId {
     lazy_static::lazy_static! {
         static ref EXAMPLE_DOT_COM: SourceId =
-            SourceId::for_registry(&"http://example.com".to_url().unwrap()).unwrap();
+            SourceId::for_registry(&"https://example.com".to_url().unwrap()).unwrap();
     }
     *EXAMPLE_DOT_COM
 }
@@ -228,15 +255,30 @@ pub fn pkg_loc(name: &str, loc: &str) -> Summary {
     .unwrap()
 }
 
+pub fn remove_dep(sum: &Summary, ind: usize) -> Summary {
+    let mut deps = sum.dependencies().to_vec();
+    deps.remove(ind);
+    // note: more things will need to be copied over in the future, but it works for now.
+    Summary::new(
+        sum.package_id(),
+        deps,
+        &BTreeMap::<String, Vec<String>>::new(),
+        sum.links().map(|a| a.as_str()),
+        sum.namespaced_features(),
+    )
+    .unwrap()
+}
+
 pub fn dep(name: &str) -> Dependency {
     dep_req(name, "*")
 }
 pub fn dep_req(name: &str, req: &str) -> Dependency {
     Dependency::parse_no_deprecated(name, Some(req), registry_loc()).unwrap()
 }
-pub fn dep_req_kind(name: &str, req: &str, kind: Kind) -> Dependency {
+pub fn dep_req_kind(name: &str, req: &str, kind: Kind, public: bool) -> Dependency {
     let mut dep = dep_req(name, req);
     dep.set_kind(kind);
+    dep.set_public(public);
     dep
 }
 
@@ -281,9 +323,12 @@ impl fmt::Debug for PrettyPrintRegistry {
             } else {
                 write!(f, "pkg!((\"{}\", \"{}\") => [", s.name(), s.version())?;
                 for d in s.dependencies() {
-                    if d.kind() == Kind::Normal && &d.version_req().to_string() == "*" {
+                    if d.kind() == Kind::Normal
+                        && &d.version_req().to_string() == "*"
+                        && !d.is_public()
+                    {
                         write!(f, "dep(\"{}\"),", d.name_in_toml())?;
-                    } else if d.kind() == Kind::Normal {
+                    } else if d.kind() == Kind::Normal && !d.is_public() {
                         write!(
                             f,
                             "dep_req(\"{}\", \"{}\"),",
@@ -293,14 +338,15 @@ impl fmt::Debug for PrettyPrintRegistry {
                     } else {
                         write!(
                             f,
-                            "dep_req_kind(\"{}\", \"{}\", {}),",
+                            "dep_req_kind(\"{}\", \"{}\", {}, {}),",
                             d.name_in_toml(),
                             d.version_req(),
                             match d.kind() {
                                 Kind::Development => "Kind::Development",
                                 Kind::Build => "Kind::Build",
                                 Kind::Normal => "Kind::Normal",
-                            }
+                            },
+                            d.is_public()
                         )?;
                     }
                 }
@@ -325,8 +371,8 @@ fn meta_test_deep_pretty_print_registry() {
                 pkg!(("bar", "2.0.0") => [dep_req("baz", "=1.0.1")]),
                 pkg!(("baz", "1.0.2") => [dep_req("other", "2")]),
                 pkg!(("baz", "1.0.1")),
-                pkg!(("cat", "1.0.2") => [dep_req_kind("other", "2", Kind::Build)]),
-                pkg!(("cat", "1.0.2") => [dep_req_kind("other", "2", Kind::Development)]),
+                pkg!(("cat", "1.0.2") => [dep_req_kind("other", "2", Kind::Build, false)]),
+                pkg!(("cat", "1.0.3") => [dep_req_kind("other", "2", Kind::Development, false)]),
                 pkg!(("dep_req", "1.0.0")),
                 pkg!(("dep_req", "2.0.0")),
             ])
@@ -338,8 +384,8 @@ fn meta_test_deep_pretty_print_registry() {
          pkg!((\"bar\", \"2.0.0\") => [dep_req(\"baz\", \"= 1.0.1\"),]),\
          pkg!((\"baz\", \"1.0.2\") => [dep_req(\"other\", \"^2\"),]),\
          pkg!((\"baz\", \"1.0.1\")),\
-         pkg!((\"cat\", \"1.0.2\") => [dep_req_kind(\"other\", \"^2\", Kind::Build),]),\
-         pkg!((\"cat\", \"1.0.2\") => [dep_req_kind(\"other\", \"^2\", Kind::Development),]),\
+         pkg!((\"cat\", \"1.0.2\") => [dep_req_kind(\"other\", \"^2\", Kind::Build, false),]),\
+         pkg!((\"cat\", \"1.0.3\") => [dep_req_kind(\"other\", \"^2\", Kind::Development, false),]),\
          pkg!((\"dep_req\", \"1.0.0\")),\
          pkg!((\"dep_req\", \"2.0.0\")),]"
     )
@@ -389,7 +435,14 @@ pub fn registry_strategy(
     let max_deps = max_versions * (max_crates * (max_crates - 1)) / shrinkage;
 
     let raw_version_range = (any::<Index>(), any::<Index>());
-    let raw_dependency = (any::<Index>(), any::<Index>(), raw_version_range, 0..=1);
+    let raw_dependency = (
+        any::<Index>(),
+        any::<Index>(),
+        raw_version_range,
+        0..=1,
+        Just(false),
+        // TODO: ^ this needs to be set back to `any::<bool>()` and work before public & private dependencies can stabilize
+    );
 
     fn order_index(a: Index, b: Index, size: usize) -> (usize, usize) {
         let (a, b) = (a.index(size), b.index(size));
@@ -398,89 +451,101 @@ pub fn registry_strategy(
 
     let list_of_raw_dependency = vec(raw_dependency, ..=max_deps);
 
-    (list_of_crates_with_versions, list_of_raw_dependency).prop_map(
-        |(crate_vers_by_name, raw_dependencies)| {
-            let list_of_pkgid: Vec<_> = crate_vers_by_name
-                .iter()
-                .flat_map(|(name, vers)| vers.iter().map(move |x| ((name.as_str(), &x.0), x.1)))
-                .collect();
-            let len_all_pkgid = list_of_pkgid.len();
-            let mut dependency_by_pkgid = vec![vec![]; len_all_pkgid];
-            for (a, b, (c, d), k) in raw_dependencies {
-                let (a, b) = order_index(a, b, len_all_pkgid);
-                let ((dep_name, _), _) = list_of_pkgid[a];
-                if (list_of_pkgid[b].0).0 == dep_name {
-                    continue;
-                }
-                let s = &crate_vers_by_name[dep_name];
-                let s_last_index = s.len() - 1;
-                let (c, d) = order_index(c, d, s.len());
+    // By default a package depends only on other packages that have a smaller name,
+    // this helps make sure that all things in the resulting index are DAGs.
+    // If this is true then the DAG is maintained with grater instead.
+    let reverse_alphabetical = any::<bool>().no_shrink();
 
-                dependency_by_pkgid[b].push(dep_req_kind(
-                    &dep_name,
-                    &if c == 0 && d == s_last_index {
-                        "*".to_string()
-                    } else if c == 0 {
-                        format!("<={}", s[d].0)
-                    } else if d == s_last_index {
-                        format!(">={}", s[c].0)
-                    } else if c == d {
-                        format!("={}", s[c].0)
-                    } else {
-                        format!(">={}, <={}", s[c].0, s[d].0)
-                    },
-                    match k {
-                        0 => Kind::Normal,
-                        1 => Kind::Build,
-                        // => Kind::Development, // Development has not impact so don't gen
-                        _ => panic!("bad index for Kind"),
-                    },
-                ))
-            }
-
-            PrettyPrintRegistry(
-                list_of_pkgid
-                    .into_iter()
-                    .zip(dependency_by_pkgid.into_iter())
-                    .map(|(((name, ver), allow_deps), deps)| {
-                        pkg_dep(
-                            (name, ver).to_pkgid(),
-                            if !allow_deps {
-                                vec![dep_req("bad", "*")]
-                            } else {
-                                let mut deps = deps;
-                                deps.sort_by_key(|d| d.name_in_toml());
-                                deps.dedup_by_key(|d| d.name_in_toml());
-                                deps
-                            },
-                        )
-                    })
-                    .collect(),
-            )
-        },
+    (
+        list_of_crates_with_versions,
+        list_of_raw_dependency,
+        reverse_alphabetical,
     )
+        .prop_map(
+            |(crate_vers_by_name, raw_dependencies, reverse_alphabetical)| {
+                let list_of_pkgid: Vec<_> = crate_vers_by_name
+                    .iter()
+                    .flat_map(|(name, vers)| vers.iter().map(move |x| ((name.as_str(), &x.0), x.1)))
+                    .collect();
+                let len_all_pkgid = list_of_pkgid.len();
+                let mut dependency_by_pkgid = vec![vec![]; len_all_pkgid];
+                for (a, b, (c, d), k, p) in raw_dependencies {
+                    let (a, b) = order_index(a, b, len_all_pkgid);
+                    let (a, b) = if reverse_alphabetical { (b, a) } else { (a, b) };
+                    let ((dep_name, _), _) = list_of_pkgid[a];
+                    if (list_of_pkgid[b].0).0 == dep_name {
+                        continue;
+                    }
+                    let s = &crate_vers_by_name[dep_name];
+                    let s_last_index = s.len() - 1;
+                    let (c, d) = order_index(c, d, s.len());
+
+                    dependency_by_pkgid[b].push(dep_req_kind(
+                        dep_name,
+                        &if c == 0 && d == s_last_index {
+                            "*".to_string()
+                        } else if c == 0 {
+                            format!("<={}", s[d].0)
+                        } else if d == s_last_index {
+                            format!(">={}", s[c].0)
+                        } else if c == d {
+                            format!("={}", s[c].0)
+                        } else {
+                            format!(">={}, <={}", s[c].0, s[d].0)
+                        },
+                        match k {
+                            0 => Kind::Normal,
+                            1 => Kind::Build,
+                            // => Kind::Development, // Development has no impact so don't gen
+                            _ => panic!("bad index for Kind"),
+                        },
+                        p,
+                    ))
+                }
+
+                PrettyPrintRegistry(
+                    list_of_pkgid
+                        .into_iter()
+                        .zip(dependency_by_pkgid.into_iter())
+                        .map(|(((name, ver), allow_deps), deps)| {
+                            pkg_dep(
+                                (name, ver).to_pkgid(),
+                                if !allow_deps {
+                                    vec![dep_req("bad", "*")]
+                                } else {
+                                    let mut deps = deps;
+                                    deps.sort_by_key(|d| d.name_in_toml());
+                                    deps.dedup_by_key(|d| d.name_in_toml());
+                                    deps
+                                },
+                            )
+                        })
+                        .collect(),
+                )
+            },
+        )
 }
 
 /// This test is to test the generator to ensure
 /// that it makes registries with large dependency trees
-///
-/// This is a form of randomized testing, if you are unlucky it can fail.
-/// A failure on it's own is not a big dael. If you did not change the
-/// `registry_strategy` then feel free to retry without concern.
 #[test]
 fn meta_test_deep_trees_from_strategy() {
     let mut dis = [0; 21];
 
     let strategy = registry_strategy(50, 20, 60);
+    let mut test_runner = TestRunner::deterministic();
     for _ in 0..128 {
         let PrettyPrintRegistry(input) = strategy
-            .new_tree(&mut TestRunner::default())
+            .new_tree(&mut TestRunner::new_with_rng(
+                Default::default(),
+                test_runner.new_rng(),
+            ))
             .unwrap()
             .current();
         let reg = registry(input.clone());
         for this in input.iter().rev().take(10) {
             let res = resolve(
-                &pkg_id("root"),
+                pkg_id("root"),
                 vec![dep_req(&this.name(), &format!("={}", this.version()))],
                 &reg,
             );
@@ -502,24 +567,24 @@ fn meta_test_deep_trees_from_strategy() {
 
 /// This test is to test the generator to ensure
 /// that it makes registries that include multiple versions of the same library
-///
-/// This is a form of randomized testing, if you are unlucky it can fail.
-/// A failure on its own is not a big deal. If you did not change the
-/// `registry_strategy` then feel free to retry without concern.
 #[test]
 fn meta_test_multiple_versions_strategy() {
     let mut dis = [0; 10];
 
     let strategy = registry_strategy(50, 20, 60);
+    let mut test_runner = TestRunner::deterministic();
     for _ in 0..128 {
         let PrettyPrintRegistry(input) = strategy
-            .new_tree(&mut TestRunner::default())
+            .new_tree(&mut TestRunner::new_with_rng(
+                Default::default(),
+                test_runner.new_rng(),
+            ))
             .unwrap()
             .current();
         let reg = registry(input.clone());
         for this in input.iter().rev().take(10) {
             let res = resolve(
-                &pkg_id("root"),
+                pkg_id("root"),
                 vec![dep_req(&this.name(), &format!("={}", this.version()))],
                 &reg,
             );
